@@ -53,6 +53,9 @@ public:
     ~Private();
     
     void updateSize();
+    void updateDiagram() const;
+    ChartType effectiveChartType() const;
+    bool isValidDataPoint( const QPoint &point ) const;
     QVariant data( const CellRegion &region, int index ) const;
     
     DataSet *parent;
@@ -138,6 +141,43 @@ void DataSet::Private::updateSize()
     }
 }
 
+/**
+ * Returns the effective chart type of this data set, i.e.
+ * returns the chart type of the diagram this data set is
+ * attached to if no chart type is set, or otherwise this data
+ * set's chart type.
+ */
+ChartType DataSet::Private::effectiveChartType() const
+{
+    if ( attachedAxis && chartType == LastChartType )
+        return attachedAxis->plotArea()->chartType();
+    return chartType;
+}
+
+void DataSet::Private::updateDiagram() const
+{
+    if ( kdDiagram && kdDataSetNumber >= 0 && size > 0 ) {
+	// FIXME: This should be done in a more proper way.
+	// The problem here is that line diagrams don't use the brush
+	// to paint their lines, but the pen. We on the other hand set
+	// the data set color on the brush, not the pen.
+        if ( effectiveChartType() == LineChartType )
+            kdDiagram->setPen( kdDataSetNumber, brush.color() );
+        else
+            kdDiagram->setPen( kdDataSetNumber, pen );
+        kdDiagram->setBrush( kdDataSetNumber, brush );
+    }
+}
+
+bool DataSet::Private::isValidDataPoint( const QPoint &point ) const {
+    if ( point.y() < 0 ) return false;
+    if ( point.x() < 0 ) return false;
+    // We can't point to horizontal and vertical header data at the same time
+    if ( point.x() == 0 && point.y() == 0 )
+        return false;
+    return true;
+}
+
 QVariant DataSet::Private::data( const CellRegion &region, int index ) const
 {
     if ( !region.isValid() )
@@ -147,19 +187,34 @@ QVariant DataSet::Private::data( const CellRegion &region, int index ) const
     if ( !model )
         return QVariant();
     
+    // The result
     QVariant data;
-        
+    
+    // Convert the given index in this dataset to a data point in the source model
     QPoint dataPoint = region.pointAtIndex( index );
-    if ( dataPoint.x() > 0 ) {
-        if ( dataPoint.y() > 0 )
-            data = model->data( model->index( dataPoint.y() - 1,
-					      dataPoint.x() - 1 ) );
-        else if ( dataPoint.y() == 0 )
-            data = model->headerData( dataPoint.x() - 1, Qt::Horizontal );
-    }
-    else if ( dataPoint.x() == 0 ) {
-        if ( dataPoint.y() > 0 )
-            data = model->headerData( dataPoint.y() - 1, Qt::Vertical );
+
+    const bool verticalHeaderData = dataPoint.x() == 0;
+    const bool horizontalHeaderData = dataPoint.y() == 0;
+
+    // Check if the data point is valid
+    const bool validDataPoint = isValidDataPoint( dataPoint );
+    Q_ASSERT( validDataPoint );
+    if ( !validDataPoint )
+        return QVariant();    
+
+    // The top-left point is (1,1). (0,y) or (x,0) refers to header data.
+    const int row = dataPoint.y() - 1;
+    const int col = dataPoint.x() - 1;
+
+    if ( verticalHeaderData )
+        data = model->headerData( row, Qt::Vertical );
+    else if ( horizontalHeaderData )
+        data = model->headerData( col, Qt::Horizontal );
+    else {
+        const QModelIndex &index = model->index( row, col );
+        Q_ASSERT( index.isValid() );
+        if ( index.isValid() )
+            data = model->data( index );
     }
     
     return data;
@@ -348,8 +403,7 @@ QBrush DataSet::brush() const
 void DataSet::setPen( const QPen &pen )
 {
     d->pen = pen;
-    if ( !d->blockSignals && d->kdDiagram && d->kdDataSetNumber >= 0 && size() > 0 )
-        d->kdDiagram->setPen( d->kdDataSetNumber, pen );
+    d->updateDiagram();
     if ( !d->blockSignals && d->attachedAxis )
         d->attachedAxis->update();
 }
@@ -357,8 +411,7 @@ void DataSet::setPen( const QPen &pen )
 void DataSet::setBrush( const QBrush &brush )
 {
     d->brush = brush;
-    if ( !d->blockSignals && d->kdDiagram && d->kdDataSetNumber >= 0 && size() > 0 )
-        d->kdDiagram->setBrush( d->kdDataSetNumber, brush );
+    d->updateDiagram();
     if ( !d->blockSignals && d->attachedAxis )
         d->attachedAxis->update();
 }
@@ -694,12 +747,18 @@ int DataSet::dimension() const
     case RadarChartType:
     case SurfaceChartType:
 	return 1;
+
     case ScatterChartType:
     case GanttChartType:
     case RingChartType:
 	return 2;
+
     case BubbleChartType:
 	return 3;
+
+    case StockChartType:
+        return 4;
+
 	// We can only determine the dimension if
 	// a chart type is set
     case LastChartType:
@@ -728,10 +787,7 @@ int DataSet::kdDataSetNumber() const
 void DataSet::setKdDataSetNumber( int number )
 {
     d->kdDataSetNumber = number;
-    if ( d->kdDiagram && d->kdDataSetNumber >= 0 && size() > 0 ) {
-        d->kdDiagram->setBrush( d->kdDataSetNumber, d->brush );
-        d->kdDiagram->setPen( d->kdDataSetNumber, d->pen );
-    }
+    d->updateDiagram();
 }
 
 void DataSet::setKdChartModel( KDChartModel *model )
