@@ -135,22 +135,30 @@ QStringList Schedule::state() const
 {
     QStringList lst;
     if ( m_deleted )
-        lst << i18n( "Deleted" );
+        lst << SchedulingState::deleted();
     if ( notScheduled )
-        lst << i18n( "Not scheduled" );
+        lst << SchedulingState::notScheduled();
     if ( schedulingError )
-        lst << i18n( "Cannot fullfill constraints" );
+        lst << SchedulingState::constraintsNotMet();
     if ( resourceError )
-        lst << i18n( "No resource allocated" );
+        lst << SchedulingState::resourceNotAllocated();
     if ( resourceNotAvailable )
-        lst << i18n( "Resource not available" );
+        lst << SchedulingState::resourceNotAvailable();
     if ( resourceOverbooked )
-        lst << i18n( "Resource overbooked" );
+        lst << SchedulingState::resourceOverbooked();
     if ( lst.isEmpty() )
-        lst << i18n( "Scheduled" );
+        lst << SchedulingState::scheduled();
     return lst;
 }
 
+bool Schedule::isBaselined() const
+{
+    if ( m_parent ) {
+        return m_parent->isBaselined();
+    }
+    return false;
+}
+
 bool Schedule::usePert() const
 {
     if ( m_parent ) {
@@ -167,15 +175,23 @@ bool Schedule::allowOverbooking() const
     return false;
 }
 
+bool Schedule::checkExternalAppointments() const
+{
+    if ( m_parent ) {
+        return m_parent->checkExternalAppointments();
+    }
+    return false;
+}
+
 void Schedule::setScheduled( bool on )
 {
     notScheduled = !on;
     changed( this );
 }
 
-TimeInterval Schedule::available( const DateTime &, const TimeInterval &interval ) const
+Duration Schedule::effort( const DateTimeInterval &interval ) const
 {
-    return TimeInterval( interval.first, interval.second );
+    return interval.second - interval.first;
 }
 
 DateTimeInterval Schedule::available( const DateTimeInterval &interval ) const
@@ -197,7 +213,7 @@ void Schedule::calcResourceOverbooked()
 {
     resourceOverbooked = false;
     foreach( Appointment *a, m_appointments ) {
-        if ( a->resource() ->isOverbooked( a->startTime().dateTime(), a->endTime().dateTime() ) ) {
+        if ( a->resource() ->isOverbooked( a->startTime(), a->endTime() ) ) {
             resourceOverbooked = true;
             break;
         }
@@ -208,7 +224,7 @@ QStringList Schedule::overbookedResources() const
 {
     QStringList rl;
     foreach( Appointment *a, m_appointments ) {
-        if ( a->resource() ->isOverbooked( a->startTime().dateTime(), a->endTime().dateTime() ) ) {
+        if ( a->resource() ->isOverbooked( a->startTime(), a->endTime() ) ) {
             rl += a->resource() ->resource() ->name();
         }
     }
@@ -225,7 +241,7 @@ QList<Resource*> Schedule::resources() const
     return QList<Resource*>();
 }
 
-bool Schedule::loadXML( const KoXmlElement &sch )
+bool Schedule::loadXML( const KoXmlElement &sch, XMLLoaderObject & )
 {
     m_name = sch.attribute( "name" );
     setType( sch.attribute( "type" ) );
@@ -276,6 +292,7 @@ void Schedule::insertBackwardNode( Node *node )
 bool Schedule::attatch( Appointment *appointment )
 {
     int mode = appointment->calculationMode();
+    //kDebug()<<appointment<<mode;
     if ( mode == Scheduling ) {
         if ( m_appointments.indexOf( appointment ) != -1 ) {
             kError() << "Appointment already exists" << endl;
@@ -401,9 +418,19 @@ Appointment Schedule::appointmentIntervals( int which ) const
 }
 
 
+EffortCostMap Schedule::bcwsPrDay() const
+{
+    //kDebug()<<m_name<<m_appointments;
+    EffortCostMap ec;
+    foreach ( Appointment *a, m_appointments ) {
+        ec += a->plannedPrDay( a->startTime().date(), a->endTime().date() );
+    }
+    return ec;
+}
+
 EffortCostMap Schedule::plannedEffortCostPrDay( const QDate &start, const QDate &end ) const
 {
-    //kDebug()<<m_name;
+    //kDebug()<<m_name<<m_appointments;
     EffortCostMap ec;
     QListIterator<Appointment*> it( m_appointments );
     while ( it.hasNext() ) {
@@ -446,10 +473,10 @@ Duration Schedule::plannedEffortTo( const QDate &date ) const
     return eff;
 }
 
-double Schedule::plannedCost() const
+EffortCost Schedule::plannedCost() const
 {
     //kDebug();
-    double c = 0;
+    EffortCost c;
     QListIterator<Appointment*> it( m_appointments );
     while ( it.hasNext() ) {
         c += it.next() ->plannedCost();
@@ -479,6 +506,22 @@ double Schedule::plannedCostTo( const QDate &date ) const
     return c;
 }
 
+void Schedule::addLog( Schedule::Log &log )
+{
+    if ( m_parent ) {
+        m_parent->addLog( log );
+    }
+}
+
+QString Schedule::Log::formatMsg() const
+ {
+    QString s;
+    s += node ? QString( "%1 " ).arg( node->name(), -8 ) : "";
+    s += resource ? QString( "%1 ").arg(resource->name(), -8 ) : "";
+    s += message;
+    return s;
+}
+
 //-------------------------------------------------
 NodeSchedule::NodeSchedule()
         : Schedule(),
@@ -559,7 +602,7 @@ bool NodeSchedule::loadXML( const KoXmlElement &sch, XMLLoaderObject &status )
 {
     //kDebug();
     QString s;
-    Schedule::loadXML( sch );
+    Schedule::loadXML( sch, status );
     s = sch.attribute( "earlystart" );
     if ( s.isEmpty() ) { // try version < 0.6
         s = sch.attribute( "earlieststart" );
@@ -694,6 +737,45 @@ QStringList NodeSchedule::resourceNameList() const
     return rl;
 }
 
+void NodeSchedule::logError( const QString &msg, int phase )
+{
+    Schedule::Log log( m_node, Log::Type_Error, msg, phase );
+    if ( m_parent ) {
+        m_parent->addLog( log );
+    } else {
+        addLog( log );
+    }
+}
+
+void NodeSchedule::logWarning( const QString &msg, int phase )
+{
+    Schedule::Log log( m_node, Log::Type_Warning, msg, phase );
+    if ( m_parent ) {
+        m_parent->addLog( log );
+    } else {
+        addLog( log );
+    }
+}
+
+void NodeSchedule::logInfo( const QString &msg, int phase )
+{
+    Schedule::Log log( m_node, Log::Type_Info, msg, phase );
+    if ( m_parent ) {
+        m_parent->addLog( log );
+    } else {
+        addLog( log );
+    }
+}
+
+void NodeSchedule::logDebug( const QString &msg, int phase )
+{
+    Schedule::Log log( m_node, Log::Type_Debug, msg, phase );
+    if ( m_parent ) {
+        m_parent->addLog( log );
+    } else {
+        addLog( log );
+    }
+}
 
 //-----------------------------------------------
 ResourceSchedule::ResourceSchedule()
@@ -706,7 +788,8 @@ ResourceSchedule::ResourceSchedule()
 ResourceSchedule::ResourceSchedule( Resource *resource, const QString& name, Schedule::Type type, long id )
         : Schedule( name, type, id ),
         m_resource( resource ),
-        m_parent( 0 )
+        m_parent( 0 ),
+        m_nodeSchedule( 0 )
 {
     //kDebug()<<"resource:"<<resource->name();
 }
@@ -714,7 +797,8 @@ ResourceSchedule::ResourceSchedule( Resource *resource, const QString& name, Sch
 ResourceSchedule::ResourceSchedule( Schedule *parent, Resource *resource )
         : Schedule( parent ),
         m_resource( resource ),
-        m_parent( parent )
+        m_parent( parent ),
+        m_nodeSchedule( 0 )
 {
     //kDebug()<<"resource:"<<resource->name();
 }
@@ -772,21 +856,21 @@ bool ResourceSchedule::isOverbooked() const
     return false;
 }
 
-bool ResourceSchedule::isOverbooked( const DateTime &start, const DateTime &end ) const
+bool ResourceSchedule::isOverbooked( const KDateTime &start, const KDateTime &end ) const
 {
     if ( m_resource == 0 )
         return false;
     //kDebug()<<start.toString()<<" -"<<end.toString();
     Appointment a = appointmentIntervals();
-    foreach ( AppointmentInterval *i, a.intervals() ) {
-        if ( ( !end.isValid() || i->startTime() < end ) &&
-                ( !start.isValid() || i->endTime() > start ) ) {
-            if ( i->load() > m_resource->units() ) {
+    foreach ( const AppointmentInterval &i, a.intervals() ) {
+        if ( ( !end.isValid() || i.startTime() < end ) &&
+                ( !start.isValid() || i.endTime() > start ) ) {
+            if ( i.load() > m_resource->units() ) {
                 //kDebug()<<m_name<<" overbooked";
                 return true;
             }
         }
-        if ( i->startTime() >= end )
+        if ( i.startTime() >= end )
             break;
     }
     //kDebug()<<m_name<<" not overbooked";
@@ -798,10 +882,34 @@ double ResourceSchedule::normalRatePrHour() const
     return m_resource ? m_resource->normalRate() : 0.0;
 }
 
-TimeInterval ResourceSchedule::available( const DateTime &date, const TimeInterval &i ) const
+//TODO change to return the booked effort
+Duration ResourceSchedule::effort( const DateTimeInterval &interval ) const
 {
-    DateTimeInterval dti = available( DateTimeInterval( DateTime( date.date(), i.first ), DateTime( date.date(), i.second ) ) );
-    return TimeInterval( dti.first.time(), dti.second.time() );
+    Duration eff = interval.second - interval.first;
+    if ( allowOverbooking() ) {
+        return eff;
+    }
+    Appointment a;
+    if ( checkExternalAppointments() ) {
+        a.setIntervals( m_resource->externalAppointments() );
+    }
+    a.merge( appointmentIntervals( m_calculationMode ) );
+    if ( a.isEmpty() || a.startTime() >= interval.second || a.endTime() <= interval.first ) {
+        return eff;
+    }
+    foreach ( const AppointmentInterval &i, a.intervals() ) {
+        if ( interval.second <= i.startTime() ) {
+            break;
+        }
+        if ( interval.first >= i.startTime() ) {
+            DateTime et = i.endTime() < interval.second ? i.endTime() : interval.second;
+            eff -= ( et - interval.first ) * ((double)i.load()/100.0 );
+        } else {
+            DateTime et = i.endTime() < interval.second ? i.endTime() : interval.second;
+            eff -= ( et - i.startTime() ) * ((double)i.load()/100.0 );
+        }
+    }
+    return eff;
 }
 
 DateTimeInterval ResourceSchedule::available( const DateTimeInterval &interval ) const
@@ -810,24 +918,28 @@ DateTimeInterval ResourceSchedule::available( const DateTimeInterval &interval )
     if ( allowOverbooking() ) {
         return DateTimeInterval( interval.first, interval.second );
     }
-    Appointment a = appointmentIntervals( m_calculationMode );
+    Appointment a;
+    if ( checkExternalAppointments() ) {
+        a.setIntervals( m_resource->externalAppointments() );
+    }
+    a.merge( appointmentIntervals( m_calculationMode ) );
     if ( a.isEmpty() || a.startTime() >= interval.second || a.endTime() <= interval.first ) {
         //kDebug()<<this<<"id="<<m_id<<"Mode="<<m_calculationMode<<""<<interval.first<<","<<interval.second<<" FREE";
         return DateTimeInterval( interval.first, interval.second );
     }
-    foreach ( AppointmentInterval *i, a.intervals() ) {
-        if ( i->startTime() < interval.second && i->endTime() > interval.first ) {
-            DateTime t = i->startTime();
+    foreach ( const AppointmentInterval &i, a.intervals() ) {
+        if ( i.startTime() < interval.second && i.endTime() > interval.first ) {
+            DateTime t = i.startTime();
             if ( t > interval.first ) {
                 //kDebug()<<this<<"id="<<m_id<<"Mode="<<m_calculationMode<<""<<interval.first<<","<<t<<" PART";
                 return DateTimeInterval( interval.first, t );
             }
-            t = i->endTime();
+            t = i.endTime();
             if ( t < interval.second ) {
                 //kDebug()<<this<<"id="<<m_id<<"Mode="<<m_calculationMode<<""<<t<<","<<interval.second<<" PART";
                 return DateTimeInterval( t, interval.second );
             }
-            if ( i->startTime() <= interval.first && i->endTime() >= interval.second ) {
+            if ( i.startTime() <= interval.first && i.endTime() >= interval.second ) {
                 //kDebug()<<this<<"id="<<m_id<<"Mode="<<m_calculationMode<<""<<interval.first<<","<<interval.second<<" BUSY";
                 return DateTimeInterval( DateTime(), DateTime() );
             }
@@ -837,7 +949,37 @@ DateTimeInterval ResourceSchedule::available( const DateTimeInterval &interval )
     return DateTimeInterval( interval.first, interval.second );
 }
 
+void ResourceSchedule::logError( const QString &msg, int phase )
+{
+    if ( m_parent ) {
+        Schedule::Log log( m_nodeSchedule->node(), m_resource, Log::Type_Error, msg, phase );
+        m_parent->addLog( log );
+    }
+}
 
+void ResourceSchedule::logWarning( const QString &msg, int phase )
+{
+    if ( m_parent ) {
+        Schedule::Log log( m_nodeSchedule->node(), m_resource, Log::Type_Warning, msg, phase );
+        m_parent->addLog( log );
+    }
+}
+
+void ResourceSchedule::logInfo( const QString &msg, int phase )
+{
+    if ( m_parent ) {
+        Schedule::Log log( m_nodeSchedule->node(), m_resource, Log::Type_Info, msg, phase );
+        m_parent->addLog( log );
+    }
+}
+
+void ResourceSchedule::logDebug( const QString &msg, int phase )
+{
+    if ( m_parent ) {
+        Schedule::Log log( m_nodeSchedule->node(), m_resource, Log::Type_Debug, msg, phase );
+        m_parent->addLog( log );
+    }
+}
 
 //--------------------------------------
 MainSchedule::MainSchedule()
@@ -863,6 +1005,16 @@ MainSchedule::~MainSchedule()
     //kDebug()<<"("<<this<<")";
 }
 
+void MainSchedule::incProgress()
+{
+    if ( m_manager ) m_manager->incProgress();
+}
+
+bool MainSchedule::isBaselined() const
+{
+    return m_manager == 0 ? false : m_manager->isBaselined();
+}
+
 bool MainSchedule::usePert() const
 {
     return m_manager == 0 ? false : m_manager->usePert();
@@ -873,6 +1025,11 @@ bool MainSchedule::allowOverbooking() const
     return m_manager == 0 ? false : m_manager->allowOverbooking();
 }
 
+bool MainSchedule::checkExternalAppointments() const
+{
+    return m_manager == 0 ? false : m_manager->checkExternalAppointments();
+}
+
 void MainSchedule::changed( Schedule *sch )
 {
     if ( m_manager ) {
@@ -884,7 +1041,7 @@ bool MainSchedule::loadXML( const KoXmlElement &sch, XMLLoaderObject &status )
 {
     //kDebug();
     QString s;
-    Schedule::loadXML( sch );
+    Schedule::loadXML( sch, status );
 
     s = sch.attribute( "start" );
     if ( !s.isEmpty() )
@@ -894,6 +1051,7 @@ bool MainSchedule::loadXML( const KoXmlElement &sch, XMLLoaderObject &status )
         endTime = DateTime::fromString( s, status.projectSpec() );
     
     duration = Duration::fromString( sch.attribute( "duration" ) );
+    schedulingError = sch.attribute( "scheduling-conflict", "0" ).toInt();
 
     KoXmlNode n = sch.firstChild();
     for ( ; ! n.isNull(); n = n.nextSibling() ) {
@@ -929,13 +1087,13 @@ bool MainSchedule::loadXML( const KoXmlElement &sch, XMLLoaderObject &status )
                     if ( e2.tagName() != "node" ) {
                         continue;
                     }
-                    kDebug()<<"node"<<endl;
+                    kDebug()<<"node";
                     QString s = e2.attribute( "id" );
                     Node *node = status.project().findNode( s );
                     if ( node ) {
                         lst.append( node );
                     } else {
-                        kError()<<"Failed to find node id="<<s<<endl;
+                        kError()<<"Failed to find node id="<<s;
                     }
                 }
                 m_pathlists.append( lst );
@@ -953,6 +1111,7 @@ void MainSchedule::saveXML( QDomElement &element ) const
     element.setAttribute( "start", startTime.toString( KDateTime::ISODate ) );
     element.setAttribute( "end", endTime.toString( KDateTime::ISODate ) );
     element.setAttribute( "duration", duration.toString() );
+    element.setAttribute( "scheduling-conflict", schedulingError );
     
     if ( ! m_pathlists.isEmpty() ) {
         QDomElement lists = element.ownerDocument().createElement( "criticalpath-list" );
@@ -1025,6 +1184,16 @@ bool MainSchedule::recalculate() const
     return m_manager == 0 ? false : m_manager->recalculate();
 }
 
+DateTime MainSchedule::recalculateFrom() const
+{
+    return m_manager == 0 ? DateTime() : m_manager->recalculateFrom();
+}
+
+long MainSchedule::parentScheduleId() const
+{
+    return m_manager == 0 ? -2 : m_manager->parentScheduleId();
+}
+
 void MainSchedule::clearCriticalPathList()
 {
     m_pathlists.clear();
@@ -1056,15 +1225,58 @@ void MainSchedule::addCriticalPathNode( Node *node )
     m_currentCriticalPath->append( node );
 }
 
+QList<Schedule::Log> MainSchedule::logs()
+{
+    return m_log;
+}
+
+const QList<Schedule::Log> &MainSchedule::logs() const
+{
+    return m_log;
+}
+
+void MainSchedule::addLog( Schedule::Log &log )
+{
+    if ( log.phase == -1 && ! m_log.isEmpty() ) {
+        log.phase = m_log.last().phase;
+    }
+    m_log.append( log );
+}
+
+//static
+QString MainSchedule::logSeverity( int severity )
+{
+    switch ( severity ) {
+        case Log::Type_Debug: return "Debug";//FIXME i18n( "Debug" );
+        case Log::Type_Info: return i18n( "Info" );
+        case Log::Type_Warning: return i18n( "Warning" );
+        case Log::Type_Error: return i18n( "Error" );
+        default: break;
+    }
+    return QString( "Severity %1" ).arg( severity );
+}
+
+QStringList MainSchedule::logMessages() const
+{
+    QStringList lst;
+    foreach ( Schedule::Log l, m_log ) {
+        lst << l.formatMsg();
+    }
+    return lst;
+}
+
 //-----------------------------------------
 ScheduleManager::ScheduleManager( Project &project, const QString name )
     : m_project( project),
     m_parent( 0 ),
     m_name( name ),
+    m_baselined( false ),
     m_allowOverbooking( false ),
+    m_checkExternalAppointments( true ),
     m_calculateAll( false ),
     m_usePert( false ),
     m_recalculate( false ),
+    m_schedulingDirection( false ),
     m_expected( 0 ),
     m_optimistic( 0 ),
     m_pessimistic( 0 )
@@ -1100,7 +1312,7 @@ int ScheduleManager::removeChild( const ScheduleManager *sm )
 
 void ScheduleManager::insertChild( ScheduleManager *sm, int index )
 {
-    kDebug()<<m_name<<", insert"<<sm->name()<<","<<index;
+    //kDebug()<<m_name<<", insert"<<sm->name()<<","<<index;
     if ( index == -1 ) {
         m_children.append( sm );
     } else {
@@ -1181,6 +1393,24 @@ void ScheduleManager::setName( const QString& name )
     m_project.changed( this );
 }
 
+bool ScheduleManager::isChildBaselined() const
+{
+    //kDebug()<<on;
+    foreach ( ScheduleManager *sm, m_children ) {
+        if ( sm->isBaselined() || sm->isChildBaselined() ) {
+            return true;
+        }
+    }
+    return false;
+}
+
+void ScheduleManager::setBaselined( bool on )
+{
+    //kDebug()<<on;
+    m_baselined = on;
+    m_project.changed( this );
+}
+
 void ScheduleManager::setAllowOverbooking( bool on )
 {
     //kDebug()<<on;
@@ -1194,6 +1424,18 @@ bool ScheduleManager::allowOverbooking() const
     return m_allowOverbooking;
 }
 
+bool ScheduleManager::checkExternalAppointments() const
+{
+    //kDebug()<<m_name<<"="<<m_allowOverbooking;
+    return m_checkExternalAppointments;
+}
+
+void ScheduleManager::setCheckExternalAppointments( bool on )
+{
+    //kDebug()<<m_name<<"="<<m_checkExternalAppointments;
+    m_checkExternalAppointments = on;
+}
+
 void ScheduleManager::scheduleChanged( MainSchedule *sch )
 {
     m_project.changed( sch );
@@ -1212,6 +1454,13 @@ void ScheduleManager::setCalculateAll( bool on )
      m_project.changed( this );
 }
 
+void ScheduleManager::setSchedulingDirection( bool on )
+{
+    //kDebug()<<on;
+    m_schedulingDirection = on;
+    m_project.changed( this );
+}
+
 void ScheduleManager::setDeleted( bool on )
 {
     if ( m_expected ) {
@@ -1295,6 +1544,9 @@ void ScheduleManager::setPessimistic( MainSchedule *sch )
 QStringList ScheduleManager::state() const
 {
     QStringList lst;
+    if ( isBaselined() ) {
+        return lst << i18n( "Baselined" );
+    }
     if ( m_expected == 0 && m_optimistic == 0 && m_pessimistic == 0 ) {
         return lst << i18n( "Not scheduled" );
     }
@@ -1332,6 +1584,11 @@ QList<MainSchedule*> ScheduleManager::schedules() const
     return m_schedules;
 }
 
+void ScheduleManager::incProgress()
+{
+    m_project.incProgress();
+}
+
 bool ScheduleManager::loadXML( KoXmlElement &element, XMLLoaderObject &status )
 {
     MainSchedule *sch = 0;
@@ -1352,6 +1609,9 @@ bool ScheduleManager::loadXML( KoXmlElement &element, XMLLoaderObject &status )
     m_name = element.attribute( "name" );
     m_usePert = (bool)(element.attribute( "distribution" ).toInt());
     m_allowOverbooking = (bool)(element.attribute( "overbooking" ).toInt());
+    m_checkExternalAppointments = (bool)(element.attribute( "check-external-appointments" ).toInt());
+    m_schedulingDirection = (bool)(element.attribute( "scheduling-direction" ).toInt());
+    m_baselined = (bool)(element.attribute( "baselined" ).toInt());
     KoXmlNode n = element.firstChild();
     for ( ; ! n.isNull(); n = n.nextSibling() ) {
         if ( ! n.isElement() ) {
@@ -1406,6 +1666,9 @@ void ScheduleManager::saveXML( QDomElement &element ) const
     el.setAttribute( "name", m_name );
     el.setAttribute( "distribution", m_usePert );
     el.setAttribute( "overbooking", m_allowOverbooking );
+    el.setAttribute( "check-external-appointments", m_checkExternalAppointments );
+    el.setAttribute( "scheduling-direction", m_schedulingDirection );
+    el.setAttribute( "baselined", m_baselined );
     foreach ( MainSchedule *s, schedules() ) {
         //kDebug()<<m_name<<" id="<<s->id()<<(s->isDeleted()?"  Deleted":"");
         if ( !s->isDeleted() && s->isScheduled() ) {
@@ -1428,6 +1691,9 @@ void ScheduleManager::saveWorkPackageXML( QDomElement &element, const Node &node
     el.setAttribute( "name", m_name );
     el.setAttribute( "distribution", m_usePert );
     el.setAttribute( "overbooking", m_allowOverbooking );
+    el.setAttribute( "check-external-appointments", m_checkExternalAppointments );
+    el.setAttribute( "scheduling-direction", m_schedulingDirection );
+    el.setAttribute( "baselined", m_baselined );
     if ( m_expected && ! m_expected->isDeleted() ) {
         QDomElement schs = el.ownerDocument().createElement( "schedule" );
         el.appendChild( schs );
@@ -1552,6 +1818,14 @@ void MainSchedule::printDebug( const QString& _indent )
     while ( it.hasNext() ) {
         it.next() ->printDebug( indent + "  " );
     }
+    kDebug()<< indent << "Scheduling log:"<<m_log.count();
+    if ( ! m_log.isEmpty() ) {
+        indent += "!-- ";
+        QList<Schedule::Log>::ConstIterator it;
+        for (it = m_log.constBegin(); it != m_log.constEnd(); ++it) {
+            kDebug()<< indent << logPhase( (*it).phase ) << (*it).node->name() << ((*it).resource? (*it).resource->name():"") << (*it).message;
+        }
+    }
 }
 #endif
 
