@@ -23,6 +23,16 @@
 #include <QtTest/QtTest>
 
 #include "kptproject.h"
+#include "kptdatetime.h"
+
+namespace QTest
+{
+    template<>
+            char *toString(const KPlato::DateTime &dt)
+    {
+        return toString( dt.toString() );
+    }
+}
 
 namespace KPlato
 {
@@ -36,13 +46,20 @@ private slots:
     void initTestCase();
     void cleanupTestCase();
     
-    void testProjectSetup();
     void testAddTask();
     void testTakeTask();
     void testTaskAddCmd();
     void testTaskDeleteCmd();
 
+    void schedule();
+    void scheduleFullday();
+
+    void scheduleWithExternalAppointments();
+
 private:
+    void printDebug( Project *p, Task *t ) const;
+    void printSchedulingLog( const ScheduleManager &sm ) const;
+
     Project *m_project;
     Task *m_task;
 };
