@@ -19,6 +19,9 @@
 
 #include "kptcalendareditor.h"
 
+#include "kcalendar/kdatepicker.h"
+#include "kcalendar/kdatetable.h"
+//#include "kptcalendarpanel.h"
 #include "kptcommand.h"
 #include "kptcalendarmodel.h"
 #include "kptcalendar.h"
@@ -27,25 +30,18 @@
 #include "kptproject.h"
 #include "kpttask.h"
 #include "kptdatetime.h"
+#include "kptintervaledit.h"
 
-#include <QDragEnterEvent>
 #include <QDragMoveEvent>
-#include <QDropEvent>
-#include <QMimeData>
-#include <QMenu>
-#include <QPainter>
-#include <QStyle>
 #include <QList>
-#include <QHeaderView>
-#include <QObject>
-#include <QTreeWidget>
-#include <QStringList>
 #include <QSplitter>
 #include <QVBoxLayout>
+#include <QHeaderView>
 
 #include <kaction.h>
 #include <kicon.h>
 #include <kglobal.h>
+#include <kglobalsettings.h>
 #include <klocale.h>
 #include <kcalendarsystem.h>
 #include <kactioncollection.h>
@@ -54,6 +50,7 @@
 #include <ktimezone.h>
 #include <kabc/addressee.h>
 #include <kabc/vcardconverter.h>
+#include <kmenu.h>
 
 #include <kdebug.h>
 
@@ -167,8 +164,10 @@ void CalendarTreeView::dragMoveEvent(QDragMoveEvent *event)
     event->ignore();
     QModelIndex index = indexAt( event->pos() );
     if ( ! index.isValid() ) {
-        event->accept();
-        return; // always ok to drop on main project
+            if ( model()->dropAllowed( 0, event->mimeData() ) ) {
+                event->accept();
+            }
+        return;
     }
     Calendar *c = model()->calendar( index );
     if ( c == 0 ) {
@@ -195,30 +194,126 @@ void CalendarTreeView::dragMoveEvent(QDragMoveEvent *event)
     }
 }
 
-
 //--------------------
 CalendarDayView::CalendarDayView( QWidget *parent )
-    : TreeViewBase( parent )
+    : QTableView( parent )
 {
-    header()->setContextMenuPolicy( Qt::CustomContextMenu );
+    setTabKeyNavigation( false );
+    setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Fixed );
+    horizontalHeader()->setResizeMode( QHeaderView::Stretch );
     m_model = new CalendarDayItemModel( this );
     setModel(m_model);
-    // TODO: sort on date & weekday number
-    for ( int c = 0; c < m_model->columnCount(); ++c ) {
-        QItemDelegate *delegate = m_model->createDelegate( c, this );
-        if ( delegate ) {
-            setItemDelegateForColumn( c, delegate );
+    verticalHeader()->hide();
+
+    actionSetWork = new KAction( i18n( "Work..." ), this );
+    connect( actionSetWork, SIGNAL( triggered( bool ) ), SLOT( slotSetWork() ) );
+    actionSetVacation = new KAction( i18n( "Non-working" ), this );
+    connect( actionSetVacation, SIGNAL( triggered( bool ) ), SLOT( slotSetVacation() ) );
+    actionSetUndefined = new KAction( i18n( "Undefined" ), this );
+    connect( actionSetUndefined, SIGNAL( triggered( bool ) ), SLOT( slotSetUndefined() ) );
+}
+
+QSize CalendarDayView::sizeHint() const
+{
+    QSize s = QTableView::sizeHint();
+    s.setHeight( horizontalHeader()->height() + rowHeight( 0 ) + frameWidth() * 2 );
+    return s;
+}
+
+void CalendarDayView::slotSetWork()
+{
+    kDebug();
+    if ( receivers( SIGNAL( executeCommand( QUndoCommand* ) ) ) == 0 ) {
+        return;
+    }
+    QModelIndexList lst = selectionModel()->selectedIndexes();
+    if ( lst.isEmpty() ) {
+        lst << currentIndex();
+    }
+    if ( lst.isEmpty() ) {
+        return;
+    }
+    IntervalEditDialog dlg( model()->day( lst.first() ) );
+    if ( dlg.exec() == QDialog::Accepted ) {
+        bool mod = false;
+        MacroCommand *m = new MacroCommand( i18n( "Modify Weekday State" ) );
+        foreach ( const QModelIndex &i, lst ) {
+            CalendarDay *day = model()->day( i );
+            if ( day == 0 ) {
+                continue;
+            }
+            MacroCommand *cmd = dlg.buildCommand( model()->calendar(), day );
+            if ( cmd ) {
+                mod = true;
+                m->addCommand( cmd );
+            }
+        }
+        if ( mod ) {
+            emit executeCommand( m );
+        } else {
+            delete m;
         }
     }
-    setSelectionBehavior( QAbstractItemView::SelectRows );
-    setSelectionMode( QAbstractItemView::SingleSelection );
-    setItemsExpandable( true );
-    
-    setAcceptDrops( false );
-    setDropIndicatorShown( false );
-    
-    connect( header(), SIGNAL( customContextMenuRequested ( const QPoint& ) ), this, SLOT( headerContextMenuRequested( const QPoint& ) ) );
-    connect( this, SIGNAL( activated ( const QModelIndex ) ), this, SLOT( slotActivated( const QModelIndex ) ) );
+}
+
+void CalendarDayView::slotSetVacation()
+{
+    kDebug();
+    if ( receivers( SIGNAL( executeCommand( QUndoCommand* ) ) ) == 0 ) {
+        return;
+    }
+    QModelIndexList lst = selectionModel()->selectedIndexes();
+    if ( lst.isEmpty() ) {
+        lst << currentIndex();
+    }
+    if ( lst.isEmpty() ) {
+        return;
+    }
+    bool mod = false;
+    MacroCommand *m = new MacroCommand( i18n( "Modify Weekday State" ) );
+    foreach ( const QModelIndex &i, lst ) {
+        CalendarDay *day = model()->day( i );
+        if ( day == 0 || day->state() == CalendarDay::NonWorking ) {
+            continue;
+        }
+        mod = true;
+        m->addCommand( new CalendarModifyStateCmd( model()->calendar(), day, CalendarDay::NonWorking ) );
+    }
+    if ( mod ) {
+        emit executeCommand( m );
+    } else {
+        delete m;
+    }
+}
+
+void CalendarDayView::slotSetUndefined()
+{
+    kDebug();
+    if ( receivers( SIGNAL( executeCommand( QUndoCommand* ) ) ) == 0 ) {
+        return;
+    }
+    QModelIndexList lst = selectionModel()->selectedIndexes();
+    if ( lst.isEmpty() ) {
+        lst << currentIndex();
+    }
+    if ( lst.isEmpty() ) {
+        return;
+    }
+    bool mod = false;
+    MacroCommand *m = new MacroCommand( i18n( "Modify Weekday State" ) );
+    foreach ( const QModelIndex &i, lst ) {
+        CalendarDay *day = model()->day( i );
+        if ( day == 0 || day->state() == CalendarDay::Undefined ) {
+            continue;
+        }
+        mod = true;
+        m->addCommand( new CalendarModifyStateCmd( model()->calendar(), day, CalendarDay::Undefined ) );
+    }
+    if ( mod ) {
+        emit executeCommand( m );
+    } else {
+        delete m;
+    }
 }
 
 void CalendarDayView::setCurrentCalendar( Calendar *calendar )
@@ -233,26 +328,33 @@ void CalendarDayView::slotActivated( const QModelIndex index )
 
 void CalendarDayView::headerContextMenuRequested( const QPoint &pos )
 {
-    kDebug()<<header()->logicalIndexAt(pos)<<" at"<<pos;
+//    kDebug()<<k_funcinfo<<header()->logicalIndexAt(pos)<<" at"<<pos;
 }
 
 void CalendarDayView::contextMenuEvent ( QContextMenuEvent *event )
 {
     //kDebug();
-    emit contextMenuRequested( indexAt(event->pos()), event->globalPos() );
+    KMenu menu;
+    menu.addAction( actionSetWork );
+    menu.addAction( actionSetVacation );
+    menu.addAction( actionSetUndefined );
+    
+    menu.exec( event->globalPos(), actionSetWork );
+    
+    //emit contextMenuRequested( indexAt(event->pos()), event->globalPos() );
 }
 
 void CalendarDayView::focusInEvent ( QFocusEvent *event )
 {
     //kDebug();
-    TreeViewBase::focusInEvent( event );
+    QTableView::focusInEvent( event );
     emit focusChanged();
 }
 
 void CalendarDayView::focusOutEvent ( QFocusEvent * event )
 {
     //kDebug();
-    TreeViewBase::focusInEvent( event );
+    QTableView::focusInEvent( event );
     emit focusChanged();
 }
 
@@ -260,49 +362,74 @@ void CalendarDayView::selectionChanged( const QItemSelection &sel, const QItemSe
 {
     //kDebug()<<sel.indexes().count();
     //foreach( QModelIndex i, selectionModel()->selectedIndexes() ) { kDebug()<<i.row()<<","<<i.column(); }
-    TreeViewBase::selectionChanged( sel, desel );
+    QTableView::selectionChanged( sel, desel );
     emit selectionChanged( selectionModel()->selectedIndexes() );
 }
 
 void CalendarDayView::currentChanged( const QModelIndex & current, const QModelIndex & previous )
 {
     //kDebug();
-    TreeViewBase::currentChanged( current, previous );
-    selectionModel()->select( current, QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows );
+    QTableView::currentChanged( current, previous );
+//    selectionModel()->select( current, QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows );
     emit currentChanged( current );
 }
 
 CalendarDay *CalendarDayView::selectedDay() const
 {
-    QModelIndexList lst = selectionModel()->selectedRows();
+    QModelIndexList lst = selectionModel()->selectedIndexes();
     if ( lst.count() == 1 ) {
         return model()->day( lst.first() );
     }
     return 0;
 }
 
-TimeInterval *CalendarDayView::selectedInterval() const
-{
-    QModelIndexList lst = selectionModel()->selectedRows();
-    if ( lst.count() == 1 ) {
-        return model()->interval( lst.first() );
-    }
-    return 0;
-}
-
 //-----------------------------------
 CalendarEditor::CalendarEditor( KoDocument *part, QWidget *parent )
-    : ViewBase( part, parent )
+    : ViewBase( part, parent ),
+    m_model( new DateTableDataModel( this ) )
 {
     setupGui();
     
-    QVBoxLayout * l = new QVBoxLayout( this );
+    QVBoxLayout *l = new QVBoxLayout( this );
     l->setMargin( 0 );
     QSplitter *sp = new QSplitter( this );
     l->addWidget( sp );
 
     m_calendarview = new CalendarTreeView( sp );
-    m_dayview = new CalendarDayView( sp );
+    
+    QFrame *f = new QFrame( sp );
+    l = new QVBoxLayout( f );
+    l->setMargin( 0 );
+
+    m_dayview = new CalendarDayView( f );
+    l->addWidget( m_dayview );
+    
+    sp = new QSplitter( f );
+    l->addWidget( sp );
+    m_datePicker = new KDatePicker( sp );
+    m_datePicker->setFrameStyle( QFrame::StyledPanel | QFrame::Sunken );
+    m_datePicker->dateTable()->setWeekNumbersEnabled( true );
+    m_datePicker->dateTable()->setGridEnabled( true );
+    m_datePicker->dateTable()->setSelectionMode( KDateTable::ExtendedSelection );
+    m_datePicker->dateTable()->setDateDelegate( new DateTableDateDelegate() );
+    m_datePicker->dateTable()->setModel( m_model );
+    m_datePicker->dateTable()->setPopupMenuEnabled( true );
+
+    m_calendarview->setDragDropMode( QAbstractItemView::InternalMove );
+    m_calendarview->setDropIndicatorShown( true );
+    m_calendarview->setDragEnabled ( true );
+    m_calendarview->setAcceptDrops( true );
+    m_calendarview->setAcceptDropsOnView( true );
+
+    connect( m_datePicker->dateTable(), SIGNAL( aboutToShowContextMenu( KMenu*, const QDate& ) ), SLOT( slotContextMenuDate( KMenu*, const QDate& ) ) );
+    connect( m_datePicker->dateTable(), SIGNAL( aboutToShowContextMenu( KMenu*, const QList<QDate>& ) ), SLOT( slotContextMenuDate( KMenu*, const QList<QDate>& ) ) );
+
+/*    const QDate date(2007,7,19);
+    const QColor fgColor(Qt::darkGray);
+    KDateTable::BackgroundMode bgMode = KDateTable::CircleMode;
+    const QColor bgColor( Qt::lightGray);
+    m_datePicker->dateTable()->setCustomDatePainting( date, fgColor, bgMode, bgColor );*/
+    
     
     m_calendarview->setEditTriggers( m_calendarview->editTriggers() | QAbstractItemView::EditKeyPressed );
     
@@ -315,6 +442,7 @@ CalendarEditor::CalendarEditor( KoDocument *part, QWidget *parent )
     
     connect( m_calendarview->model(), SIGNAL( executeCommand( QUndoCommand* ) ), part, SLOT( addCommand( QUndoCommand* ) ) );
     connect( m_dayview->model(), SIGNAL( executeCommand( QUndoCommand* ) ), part, SLOT( addCommand( QUndoCommand* ) ) );
+    connect( m_dayview, SIGNAL( executeCommand( QUndoCommand* ) ), part, SLOT( addCommand( QUndoCommand* ) ) );
 
     connect( m_calendarview, SIGNAL( currentChanged( QModelIndex ) ), this, SLOT( slotCurrentCalendarChanged( QModelIndex ) ) );
     connect( m_calendarview, SIGNAL( selectionChanged( const QModelIndexList ) ), this, SLOT( slotCalendarSelectionChanged( const QModelIndexList ) ) );
@@ -354,18 +482,41 @@ void CalendarEditor::setGuiActive( bool activate )
     }
 }
 
+void CalendarEditor::slotContextMenuDate( KMenu *menu, const QList<QDate> &dates )
+{
+    kDebug()<<menu<<dates;
+    if ( dates.isEmpty() ) {
+        m_currentMenuDateList << m_datePicker->date();
+    } else {
+        m_currentMenuDateList = dates;
+    }
+    menu->addAction( actionSetWork );
+    menu->addAction( actionSetVacation );
+    menu->addAction( actionSetUndefined );
+}
+
+void CalendarEditor::slotContextMenuDate( KMenu *menu, const QDate &date )
+{
+    kDebug()<<menu<<date;
+    if ( ! date.isValid() ) {
+        return;
+    }
+    m_currentMenuDateList << date;
+    menu->addAction( actionSetWork );
+    menu->addAction( actionSetVacation );
+    menu->addAction( actionSetUndefined );
+}
+
 void CalendarEditor::slotContextMenuCalendar( QModelIndex index, const QPoint& pos )
 {
     //kDebug()<<index.row()<<","<<index.column()<<":"<<pos;
     QString name;
-    if ( index.isValid() ) {
+/*    if ( index.isValid() ) {
         Calendar *a = m_calendarview->model()->calendar( index );
         if ( a ) {
             name = "calendareditor_calendar_popup";
-        } else if ( m_dayview->model()->day( index ) ) {
-            name = "calendareditor_day_popup";
         }
-    }
+    }*/
     //kDebug()<<name;
     if ( name.isEmpty() ) {
         return;
@@ -398,9 +549,12 @@ void CalendarEditor::slotCurrentCalendarChanged(  const QModelIndex & )
 {
     //kDebug()<<curr.row()<<","<<curr.column();
     m_dayview->setCurrentCalendar( currentCalendar() );
+    if ( m_model ) {
+        m_model->setCalendar( currentCalendar() );
+    }
 }
 
-void CalendarEditor::slotCalendarSelectionChanged( const QModelIndexList )
+void CalendarEditor::slotCalendarSelectionChanged( const QModelIndexList list )
 {
     //kDebug()<<list.count();
     updateActionsEnabled( true );
@@ -427,30 +581,9 @@ void CalendarEditor::updateActionsEnabled(  bool on )
     QList<Calendar *> lst = m_calendarview->selectedCalendars();
     bool one = lst.count() == 1;
     bool more = lst.count() > 1;
-    bool cal = m_calendarview->hasFocus();
-    bool day = m_dayview->hasFocus();
-    actionAddCalendar ->setEnabled( on && !more && cal );
-    actionAddSubCalendar ->setEnabled( on && one && cal );
-    actionDeleteSelection->setEnabled( on && ( one || more ) && cal );
-
-    bool o = false;
-    TimeInterval *ti = 0;
-    CalendarDay *d = m_dayview->selectedDay();
-    if ( on ) {
-        o = d == 0 ? false : d->state() == CalendarDay::Working;
-        if ( !o ) {
-            ti = m_dayview->selectedInterval();
-            if ( ti ) {
-                d = m_dayview->model()->parentDay( ti );
-                o = d->state() == CalendarDay::Working;
-            }
-        }
-    }
-    actionAddDay->setEnabled( on && day && !ti && ( d == 0 || m_dayview->model()->isDate( d ) ) );
-    actionAddWorkInterval->setEnabled( on && o && day );
-    
-    bool act = on && day && ( ti || ( d && m_dayview->model()->isDate( d ) ) );
-    actionDeleteDaySelection->setEnabled( act );
+    actionAddCalendar ->setEnabled( on && !more );
+    actionAddSubCalendar ->setEnabled( on && one );
+    actionDeleteSelection->setEnabled( on && ( one || more ) );
 }
 
 void CalendarEditor::setupGui()
@@ -473,34 +606,23 @@ void CalendarEditor::setupGui()
     actionDeleteSelection->setShortcut( KShortcut( Qt::Key_Delete ) );
     connect( actionDeleteSelection, SIGNAL( triggered( bool ) ), SLOT( slotDeleteCalendar() ) );
     
-    addAction( name, actionAddSubCalendar  );
     addAction( name, actionAddCalendar  );
+    addAction( name, actionAddSubCalendar  );
     addAction( name, actionDeleteSelection );
     
-    name = "calendareditor_day_list";
-    
-    actionAddDay   = new KAction(KIcon( "document-new" ), i18n("Add Calendar Day"), this);
-    coll->addAction("add_calendarday", actionAddDay  );
-    connect( actionAddDay , SIGNAL( triggered( bool ) ), SLOT( slotAddDay() ) );
-    
-    actionAddWorkInterval   = new KAction(KIcon( "document-new" ), i18n("Add Work Interval"), this);
-    coll->addAction("add_workinterval", actionAddWorkInterval  );
-    connect( actionAddWorkInterval , SIGNAL( triggered( bool ) ), SLOT( slotAddInterval() ) );
-    
-    actionDeleteDaySelection  = new KAction(KIcon( "edit-delete" ), i18n("Delete Selected Item"), this);
-    coll->addAction("delete_day_selection", actionDeleteDaySelection );
-    connect( actionDeleteDaySelection, SIGNAL( triggered( bool ) ), SLOT( slotDeleteDaySelection() ) );
-    
-    addAction( name, actionAddWorkInterval  );
-    addAction( name, actionAddDay  );
-    addAction( name, actionDeleteDaySelection );
-    
+    actionSetWork = new KAction( i18n( "Work..." ), this );
+    connect( actionSetWork, SIGNAL( triggered( bool ) ), SLOT( slotSetWork() ) );
+    actionSetVacation = new KAction( i18n( "Non-working" ), this );
+    connect( actionSetVacation, SIGNAL( triggered( bool ) ), SLOT( slotSetVacation() ) );
+    actionSetUndefined = new KAction( i18n( "Undefined" ), this );
+    connect( actionSetUndefined, SIGNAL( triggered( bool ) ), SLOT( slotSetUndefined() ) );
+
 }
 
 void CalendarEditor::updateReadWrite( bool readwrite )
 {
     m_calendarview->setReadWrite( readwrite );
-    m_dayview->setReadWrite( readwrite );
+//    m_dayview->setReadWrite( readwrite );
 }
 
 void CalendarEditor::slotAddCalendar ()
@@ -542,7 +664,7 @@ void CalendarEditor::slotDeleteCalendar()
 void CalendarEditor::slotAddInterval ()
 {
     //kDebug();
-    CalendarDay *parent = m_dayview->selectedDay ();
+/*    CalendarDay *parent = m_dayview->selectedDay ();
     if ( parent == 0 ) {
         TimeInterval *ti = m_dayview->selectedInterval();
         if ( ti == 0 ) {
@@ -559,13 +681,13 @@ void CalendarEditor::slotAddInterval ()
         m_dayview->setExpanded( p, true );
         m_dayview->setCurrentIndex( i );
         m_dayview->edit( i );
-    }
+    }*/
 }
 
 void CalendarEditor::slotDeleteDaySelection()
 {
     //kDebug();
-    TimeInterval *ti = m_dayview->selectedInterval();
+/*    TimeInterval *ti = m_dayview->selectedInterval();
     if ( ti != 0 ) {
         m_dayview->model()->removeInterval( ti );
         return;
@@ -573,13 +695,13 @@ void CalendarEditor::slotDeleteDaySelection()
     CalendarDay *day = m_dayview->selectedDay();
     if ( day != 0 ) {
         m_dayview->model()->removeDay( day );
-    }
+    }*/
 }
 
 void CalendarEditor::slotAddDay ()
 {
     //kDebug();
-    Calendar *c = currentCalendar();
+/*    Calendar *c = currentCalendar();
     if ( c == 0 ) {
         return;
     }
@@ -593,7 +715,108 @@ void CalendarEditor::slotAddDay ()
         m_dayview->setExpanded( p, true );
         m_dayview->setCurrentIndex( i );
         m_dayview->edit( i );
+    }*/
+}
+
+void CalendarEditor::slotSetWork()
+{
+    kDebug()<<currentCalendar()<<m_currentMenuDateList;
+    if ( currentCalendar() == 0 || m_currentMenuDateList.isEmpty() ) {
+        return;
     }
+    IntervalEditDialog dlg( currentCalendar()->findDay( m_currentMenuDateList.first() ) );
+    if ( dlg.exec() == QDialog::Accepted ) {
+        bool mod = false;
+        MacroCommand *m = new MacroCommand( i18n ( "Modify Calendar" ) );
+        foreach ( const QDate &date, m_currentMenuDateList ) {
+            CalendarDay *day = currentCalendar()->findDay( date );
+            if ( day == 0 ) {
+                mod = true;
+                day = new CalendarDay( date, CalendarDay::Working );
+                day->setIntervals( dlg.intervals() );
+                m->addCommand( new CalendarAddDayCmd( currentCalendar(), day ) );
+                if ( m_currentMenuDateList.count() == 1 ) {
+                    m->setText( i18n( "%1: Set to Working", date.toString() ) );
+                }
+            } else {
+                MacroCommand *cmd = dlg.buildCommand( currentCalendar(), day );
+                if ( cmd ) {
+                    mod = true;
+                    kDebug()<<date<<"Modify day"<<day;
+                    m->addCommand( cmd );
+                    if ( m_currentMenuDateList.count() == 1 ) {
+                        m->setText( i18n( "%1: Modify Work Interval", date.toString() ) );
+                    }
+                }
+            }
+        }
+        if ( mod ) {
+            part()->addCommand( m );
+        } else {
+            delete m;
+        }
+    }
+    m_currentMenuDateList.clear();
+}
+
+void CalendarEditor::slotSetVacation()
+{
+    kDebug()<<m_currentMenuDateList;
+    if ( m_currentMenuDateList.isEmpty() || currentCalendar() == 0 ) {
+        return;
+    }
+    bool mod = false;
+    MacroCommand *m = new MacroCommand( i18n ( "Modify Calendar" ) );
+    foreach ( const QDate &date, m_currentMenuDateList ) {
+        kDebug()<<"handle:"<<date;
+        CalendarDay *day = currentCalendar()->findDay( date );
+        if ( day == 0 ) {
+            mod = true;
+            day = new CalendarDay( date, CalendarDay::NonWorking );
+            m->addCommand( new CalendarAddDayCmd( currentCalendar(), day ) );
+            if ( m_currentMenuDateList.count() == 1 ) {
+                m->setText( i18n( "%1: Set to Non-Working", date.toString() ) );
+            }
+        } else if ( day->state() != CalendarDay::NonWorking ) {
+            mod = true;
+            m->addCommand( new CalendarModifyStateCmd( currentCalendar(), day, CalendarDay::NonWorking ) );
+            if ( m_currentMenuDateList.count() == 1 ) {
+                m->setText( i18n( "%1: Set to Non-Working", date.toString() ) );
+            }
+        }
+    }
+    if ( mod ) {
+        part()->addCommand( m );
+    } else {
+        delete m;
+    }
+    m_currentMenuDateList.clear();
+}
+
+void CalendarEditor::slotSetUndefined()
+{
+    kDebug();
+    if ( m_currentMenuDateList.isEmpty() || currentCalendar() == 0 ) {
+        return;
+    }
+    bool mod = false;
+    MacroCommand *m = new MacroCommand( i18n ( "Modify Calendar" ) );
+    foreach ( const QDate &date, m_currentMenuDateList ) {
+        CalendarDay *day = currentCalendar()->findDay( date );
+        if ( day && day->state() != CalendarDay::Undefined ) {
+            mod = true;
+            m->addCommand( new CalendarRemoveDayCmd( currentCalendar(), day ) );
+            if ( m_currentMenuDateList.count() == 1 ) {
+                m->setText( i18n( "Set %1 to Undefined", date.toString() ) );
+            }
+        }
+    }
+    if ( mod ) {
+        part()->addCommand( m );
+    } else {
+        delete m;
+    }
+    m_currentMenuDateList.clear();
 }
 
 
