@@ -29,8 +29,6 @@
 #include <QImage>
 #include <QPainter>
 #include <QSize>
-#include <QApplication>
-#include <QThread>
 #include <QDateTime>
 #include <QRect>
 #include <QRegion>
@@ -78,8 +76,6 @@ public:
     bool sizeChangedWhileLocked;
     KisPerspectiveGrid* perspectiveGrid;
 
-    KUrl uri;
-
     qint32 width;
     qint32 height;
 
@@ -126,7 +122,6 @@ KisImage::KisImage(const KisImage& rhs)
         else
             m_d->perspectiveGrid = 0;
 
-        m_d->uri = rhs.m_d->uri;
         m_d->width = rhs.m_d->width;
         m_d->height = rhs.m_d->height;
         m_d->xres = rhs.m_d->xres;
@@ -472,8 +467,8 @@ void KisImage::rotate(double radians, KoUpdater *progress)
     qint32 h = height();
     qint32 tx = qint32((w * cos(radians) - h * sin(radians) - w) / 2 + 0.5);
     qint32 ty = qint32((h * cos(radians) + w * sin(radians) - h) / 2 + 0.5);
-    w = (qint32)(width() * QABS(cos(radians)) + height() * QABS(sin(radians)) + 0.5);
-    h = (qint32)(height() * QABS(cos(radians)) + width() * QABS(sin(radians)) + 0.5);
+    w = (qint32)(width() * qAbs(cos(radians)) + height() * qAbs(sin(radians)) + 0.5);
+    h = (qint32)(height() * qAbs(cos(radians)) + width() * qAbs(sin(radians)) + 0.5);
 
     tx -= (w - width()) / 2;
     ty -= (h - height()) / 2;
@@ -512,18 +507,18 @@ void KisImage::shear(double angleX, double angleY, KoUpdater *progress)
 
 
     if (angleX != 0 || angleY != 0) {
-        double deltaY = height() * QABS(tan(angleX * pi / 180) * tan(angleY * pi / 180));
-        w = (qint32)(width() + QABS(height() * tan(angleX * pi / 180)));
+        double deltaY = height() * qAbs(tan(angleX * pi / 180) * tan(angleY * pi / 180));
+        w = (qint32)(width() + qAbs(height() * tan(angleX * pi / 180)));
         //ugly fix for the problem of having two extra pixels if only a shear along one
         //axis is done. This has to be fixed in the cropping code in KisRotateVisitor!
         if (angleX == 0 || angleY == 0)
-            h = (qint32)(height() + QABS(w * tan(angleY * pi / 180)));
+            h = (qint32)(height() + qAbs(w * tan(angleY * pi / 180)));
         else if (angleX > 0 && angleY > 0)
-            h = (qint32)(height() + QABS(w * tan(angleY * pi / 180)) - 2 * deltaY + 2);
+            h = (qint32)(height() + qAbs(w * tan(angleY * pi / 180)) - 2 * deltaY + 2);
         else if (angleX < 0 && angleY < 0)
-            h = (qint32)(height() + QABS(w * tan(angleY * pi / 180)) - 2 * deltaY + 2);
+            h = (qint32)(height() + qAbs(w * tan(angleY * pi / 180)) - 2 * deltaY + 2);
         else
-            h = (qint32)(height() + QABS(w * tan(angleY * pi / 180)));
+            h = (qint32)(height() + qAbs(w * tan(angleY * pi / 180)));
     }
 
     if (w != width() || h != height()) {
@@ -624,6 +619,7 @@ void KisImage::setResolution(double xres, double yres)
 {
     m_d->xres = xres;
     m_d->yres = yres;
+    emit(sigResolutionChanged(xres, yres));
 }
 
 QPointF KisImage::documentToPixel(const QPointF &documentCoord) const
@@ -684,92 +680,6 @@ KisPaintDeviceSP KisImage::projection()
     return projection;
 }
 
-bool KisImage::addLayer(KisLayerSP layer, KisGroupLayerSP group)
-{
-    // XXX: Temporarily for compatibility
-    if (group)
-        return addNode(layer.data(), group.data());
-    else {
-        KisNode * root = rootLayer().data();
-        return addNode(layer, root);
-    }
-}
-
-bool KisImage::addLayer(KisLayerSP layer, KisGroupLayerSP parent, KisLayerSP aboveThis)
-{
-    if (parent && aboveThis)
-        return addNode(layer.data(), parent.data(), aboveThis.data());
-    else if (parent)
-        return addNode(layer.data(), parent.data());
-    else
-        return addNode(layer.data(), rootLayer().data());
-}
-
-bool KisImage::addLayer(KisLayerSP layer,  KisGroupLayerSP parent, int index)
-{
-    if (parent)
-        return addNode(layer.data(), parent.data(), index);
-    else
-        return addNode(layer.data(), rootLayer().data(), index);
-}
-
-
-bool KisImage::removeLayer(KisLayerSP layer)
-{
-    if (KisGroupLayerSP parent = dynamic_cast<KisGroupLayer*>(layer->parent().data())) {
-
-        KisNodeSP wasAbove = layer->nextSibling();
-        KisNodeSP wasBelow = layer->prevSibling();
-
-        const bool success = removeNode(layer.data());
-
-        if (success) {
-            layer->setImage(0);
-            if (!layer->temporary() && undo()) {
-                m_d->adapter->addCommand(new KisImageLayerRemoveCommand(KisImageSP(this),
-                                         layer,
-                                         parent,
-                                         wasAbove));
-            }
-        }
-        return success;
-    }
-
-    return false;
-}
-
-bool KisImage::raiseLayer(KisLayerSP layer)
-{
-    return raiseNode(layer.data());
-}
-
-bool KisImage::lowerLayer(KisLayerSP layer)
-{
-    return lowerNode(layer.data());
-}
-
-bool KisImage::moveLayer(KisLayerSP layer, KisGroupLayerSP parent, KisLayerSP aboveThis)
-{
-    // XXX: Temporary until everything is moved to handling just nodes
-    lock();
-    KisGroupLayerSP wasParent = dynamic_cast<KisGroupLayer*>(layer->parent().data());
-    KisLayerSP wasAbove = dynamic_cast<KisLayer*>(layer->nextSibling().data());
-    if (wasParent.data() == parent.data() && wasAbove.data() == aboveThis.data())
-        return false;
-    bool success = moveNode(layer.data(), parent.data(), aboveThis.data());
-    if (success) {
-        if (undo())
-            m_d->adapter->addCommand(new KisImageLayerMoveCommand(KisImageSP(this),
-                                     layer,
-                                     wasParent,
-                                     wasAbove));
-    }
-
-    unlock();
-
-    return success;
-}
-
 qint32 KisImage::nlayers() const
 {
     QStringList list;
@@ -804,7 +714,7 @@ void KisImage::flatten()
     KisPainter gc(dst->paintDevice());
     gc.bitBlt(rc.x(), rc.y(), COMPOSITE_COPY, mergedImage(), OPACITY_OPAQUE, rc.left(), rc.top(), rc.width(), rc.height());
 
-    setRootLayer(new KisGroupLayer(this, "", OPACITY_OPAQUE));
+    setRootLayer(new KisGroupLayer(this, "root", OPACITY_OPAQUE));
 
     if (undo()) {
         m_d->adapter->beginMacro(i18n("Flatten Image"));
@@ -832,6 +742,7 @@ KisLayerSP KisImage::mergeLayer(KisLayerSP layer, const KisMetaData::MergeStrate
     if (!layer->prevSibling()) return 0;
     // XXX: this breaks if we allow free mixing of masks and layers
     KisLayerSP layer2 = dynamic_cast<KisLayer*>(layer->prevSibling().data());
+    dbgImage << "Merge " << layer << " with " << layer2;
     if (!layer2) return 0;
 
     KisPaintLayerSP newLayer = new KisPaintLayer(this, layer->name(), OPACITY_OPAQUE, colorSpace());
@@ -865,12 +776,18 @@ KisLayerSP KisImage::mergeLayer(KisLayerSP layer, const KisMetaData::MergeStrate
     scores.append(layerArea / norm);
     strategy->merge(newLayer->metaData(), srcs, scores);
 
-    removeNode(layer->prevSibling());
-    removeNode(layer);
-    addNode(newLayer, layer->parent());
+    KisNodeSP parent = layer->parent(); // parent is set to null when the layer is removed from the node
+    dbgImage << ppVar( parent );
+//     addNode(newLayer, parent, layer);
+//     removeNode(layer->prevSibling());
+//     removeNode(layer);
+
+    undoAdapter()->addCommand(new KisImageLayerAddCommand(this, newLayer, parent, layer ));
+    undoAdapter()->addCommand(new KisImageLayerRemoveCommand(this, layer->prevSibling() ));
+    undoAdapter()->addCommand(new KisImageLayerRemoveCommand(this, layer ));
 
     undoAdapter()->endMacro();
-    
+
     return newLayer;
 }
 
@@ -930,7 +847,9 @@ QImage KisImage::convertToQImage(qint32 x,
 
 QImage KisImage::convertToQImage(const QRect& r, const double xScale, const double yScale, const KoColorProfile *profile, KisSelectionSP mask)
 {
-    Q_UNUSED(mask)
+    Q_UNUSED(mask);
+
+    qDebug() << "KisImage::convertToQimage " << r << ", x scale " << xScale << ", y scale " << yScale;
 
 #ifdef __GNUC__
 #warning "KisImage::convertToQImage: Implement direct rendering of current mask onto scaled image pixels"
@@ -1002,6 +921,87 @@ QImage KisImage::convertToQImage(const QRect& r, const double xScale, const doub
     return image;
 }
 
+
+QImage KisImage::convertToQImage(const QRect& r, const QSize& scaledImageSize, const KoColorProfile *profile)
+{
+
+    if (r.isEmpty() || scaledImageSize.isEmpty()) {
+        return QImage();
+    }
+
+    qint32 imageWidth = width();
+    qint32 imageHeight = height();
+    quint32 pixelSize = colorSpace()->pixelSize();
+
+    double xScale = static_cast<double>(imageWidth) / scaledImageSize.width();
+    double yScale = static_cast<double>(imageHeight) / scaledImageSize.height();
+
+    QRect srcRect;
+
+    srcRect.setLeft(static_cast<int>(r.left() * xScale));
+    srcRect.setRight(static_cast<int>(ceil((r.right() + 1) * xScale)) - 1);
+    srcRect.setTop(static_cast<int>(r.top() * yScale));
+    srcRect.setBottom(static_cast<int>(ceil((r.bottom() + 1) * yScale)) - 1);
+
+    KisPaintDeviceSP mergedImage = m_d->rootLayer->projection();
+    QTime t;
+    t.start();
+
+    quint8 *scaledImageData = new quint8[r.width() * r.height() * pixelSize];
+
+    quint8 *imageRow = new quint8[srcRect.width() * pixelSize];
+    const qint32 imageRowX = srcRect.x();
+
+    for (qint32 y = 0; y < r.height(); ++y) {
+
+        qint32 dstY = r.y() + y;
+        qint32 dstX = r.x();
+        qint32 srcY = (dstY * imageHeight) / scaledImageSize.height();
+
+        mergedImage->readBytes(imageRow, imageRowX, srcY, srcRect.width(), 1);
+
+        quint8 *dstPixel = scaledImageData + (y * r.width() * pixelSize);
+        quint32 columnsRemaining = r.width();
+
+        while (columnsRemaining > 0) {
+
+            qint32 srcX = (dstX * imageWidth) / scaledImageSize.width();
+
+            memcpy(dstPixel, imageRow + ((srcX - imageRowX) * pixelSize), pixelSize);
+
+            ++dstX;
+            dstPixel += pixelSize;
+            --columnsRemaining;
+        }
+    }
+    dbgRender << "Time elapsed scaling image: " << t.elapsed() << endl;
+
+    delete [] imageRow;
+
+    QImage image = colorSpace()->convertToQImage(scaledImageData, r.width(), r.height(), const_cast<KoColorProfile*>(profile), KoColorConversionTransformation::IntentPerceptual);
+
+    delete [] scaledImageData;
+
+#ifdef __BIG_ENDIAN__
+    uchar * data = image.bits();
+    for (int i = 0; i < image.width() * image.height(); ++i) {
+      uchar r, g, b, a;
+      a = data[0];
+      b = data[1];
+      g = data[2];
+      r = data[3];
+      data[0] = r;
+      data[1] = g;
+      data[2] = b;
+      data[3] = a;
+      data += 4;
+    }
+#endif
+
+    return image;
+}
+
+
 KisPaintDeviceSP KisImage::mergedImage()
 {
     return m_d->rootLayer->projection();
@@ -1010,6 +1010,7 @@ KisPaintDeviceSP KisImage::mergedImage()
 void KisImage::notifyLayersChanged()
 {
     emit sigLayersChanged(rootLayer());
+    emit sigPostLayersChanged(rootLayer());
 }
 
 void KisImage::notifyPropertyChanged(KisLayerSP layer)
