@@ -65,7 +65,6 @@
 #include "dialogs/kis_dlg_adjustment_layer.h"
 #include "dialogs/kis_dlg_layer_properties.h"
 #include "dialogs/kis_dlg_generator_layer.h"
-#include "dialogs/kis_dlg_new_layer.h"
 #include "kis_doc2.h"
 #include "kis_filter_manager.h"
 #include "commands/kis_image_commands.h"
@@ -80,6 +79,7 @@
 #include "widgets/kis_meta_data_merge_strategy_chooser_widget.h"
 #include "widgets/kis_wdg_generator.h"
 #include "kis_layer_box.h"
+#include "kis_node_commands_adapter.h"
 
 KisLayerManager::KisLayerManager(KisView2 * view, KisDoc2 * doc)
         : m_view(view)
@@ -103,11 +103,13 @@ KisLayerManager::KisLayerManager(KisView2 * view, KisDoc2 * doc)
         , m_actLayerVis(false)
         , m_imgResizeToLayer(0)
         , m_activeLayer(0)
+        , m_commandsAdapter( new KisNodeCommandsAdapter( m_view ) )
 {
 }
 
 KisLayerManager::~KisLayerManager()
 {
+    delete m_commandsAdapter;
 }
 
 KisLayerSP KisLayerManager::activeLayer()
@@ -304,7 +306,7 @@ void KisLayerManager::layerCompositeOp(const KoCompositeOp* compositeOp)
     KisLayerSP layer = activeLayer();
     if (!layer) return;
 
-    m_doc->addCommand(new KisLayerCompositeOpCommand(layer, layer->compositeOp(), compositeOp));
+    m_doc->addCommand(new KisLayerCompositeOpCommand(layer, layer->compositeOpId(), compositeOp->id()));
 }
 
 // range: 0 - 100
@@ -371,24 +373,30 @@ void KisLayerManager::layerProperties()
         if (prev) dev = prev->projection();
 
         KisDlgAdjLayerProps dlg(dev, alayer->image(), alayer->filter(), alayer->name(), i18n("Filter Layer Properties"), m_view, "dlgadjlayerprops");
+        dlg.resize( dlg.minimumSizeHint() );
         KisFilterConfiguration* config = dlg.filterConfiguration();
         QString before;
-        if (config)
+        if ( config ) {
             before = config->toLegacyXML();
+        }
         if (dlg.exec() == QDialog::Accepted) {
 
             QString after;
-            if (dlg.filterConfiguration())
+            if (dlg.filterConfiguration()) {
                 after = dlg.filterConfiguration()->toLegacyXML();
-            alayer->setName(dlg.layerName());
-            KisChangeFilterCmd<KisAdjustmentLayerSP> * cmd
-            = new KisChangeFilterCmd<KisAdjustmentLayerSP>(alayer,
-                    dlg.filterConfiguration(),
-                    before,
-                    after);
-            cmd->redo();
-            m_view->undoAdapter()->addCommand(cmd);
-            m_doc->setModified(true);
+            }
+            if ( after != before ) {
+                alayer->setName(dlg.layerName());
+
+                KisChangeFilterCmd<KisAdjustmentLayerSP> * cmd
+                    = new KisChangeFilterCmd<KisAdjustmentLayerSP>(alayer,
+                                                                   dlg.filterConfiguration(),
+                                                                   before,
+                                                                   after);
+                cmd->redo();
+                m_view->undoAdapter()->addCommand(cmd);
+                m_doc->setModified(true);
+            }
         }
     } else if (KisGeneratorLayerSP alayer = KisGeneratorLayerSP(dynamic_cast<KisGeneratorLayer*>(layer.data()))) {
 
@@ -397,17 +405,23 @@ void KisLayerManager::layerProperties()
 
         QString before = alayer->generator()->toLegacyXML();
         dlg.setConfiguration(alayer->generator());
+        dlg.resize( dlg.minimumSizeHint() );
 
         if (dlg.exec() == QDialog::Accepted) {
 
-            KisChangeGeneratorCmd<KisGeneratorLayerSP> * cmd
-            = new KisChangeGeneratorCmd<KisGeneratorLayerSP>(alayer,
-                    dlg.configuration(),
-                    before,
-                    dlg.configuration()->toLegacyXML());
-            cmd->redo();
-            m_view->undoAdapter()->addCommand(cmd);
-            m_doc->setModified(true);
+            QString after = dlg.configuration()->toLegacyXML();
+            if ( after != before ) {
+                KisChangeGeneratorCmd<KisGeneratorLayerSP> * cmd
+                                    = new KisChangeGeneratorCmd<KisGeneratorLayerSP>(alayer,
+                                                                                     dlg.configuration(),
+                                                                                     before,
+                                                                                     after
+                                        );
+
+                cmd->redo();
+                m_view->undoAdapter()->addCommand(cmd);
+                m_doc->setModified(true);
+            }
 
         }
     } else {
@@ -416,8 +430,10 @@ void KisLayerManager::layerProperties()
                                   layer->compositeOp(),
                                   cs,
                                   layer->channelFlags());
+        dlg.resize( dlg.minimumSizeHint() );
+
         if (dlg.exec() == QDialog::Accepted) {
-        
+
             QBitArray newChannelFlags = dlg.getChannelFlags();
             for (int i = 0; i < newChannelFlags.size(); ++i) {
                 dbgUI << "we got flags: " << i << " is " << newChannelFlags.testBit(i);
@@ -437,7 +453,7 @@ void KisLayerManager::layerProperties()
                 QApplication::setOverrideCursor(KisCursor::waitCursor());
                 m_view->undoAdapter()->addCommand(new KisLayerPropsCommand(layer,
                                                   layer->opacity(), dlg.getOpacity(),
-                                                  layer->compositeOp(), dlg.getCompositeOp(),
+                                                  layer->compositeOpId(), dlg.getCompositeOp()->id(),
                                                   layer->name(), dlg.getName(),
                                                   oldChannelFlags, newChannelFlags));
                 QApplication::restoreOverrideCursor();
@@ -465,8 +481,8 @@ void KisLayerManager::addLayer(KisNodeSP parent, KisNodeSP above)
         QString profilename;
         KisLayerSP layer = new KisPaintLayer(img.data(), img->nextLayerName(), OPACITY_OPAQUE, img->colorSpace());
         if (layer) {
-            layer->setCompositeOp(img->colorSpace()->compositeOp(COMPOSITE_OVER));
-            img->addNode(layer.data(), parent.data(), above.data());
+            layer->setCompositeOp(COMPOSITE_OVER);
+            m_commandsAdapter->addNode(layer.data(), parent.data(), above.data());
             m_view->canvas()->update();
             m_view->layerBox()->setCurrentNode( layer );
         } else {
@@ -481,8 +497,8 @@ void KisLayerManager::addGroupLayer(KisNodeSP parent, KisNodeSP above)
     if (img) {
         KisLayerSP layer = KisLayerSP(new KisGroupLayer(img.data(), img->nextLayerName(), OPACITY_OPAQUE));
         if (layer) {
-            layer->setCompositeOp(img->colorSpace()->compositeOp(COMPOSITE_OVER));
-            img->addNode(layer.data(), parent.data(), above.data());
+            layer->setCompositeOp(COMPOSITE_OVER);
+            m_commandsAdapter->addNode(layer.data(), parent.data(), above.data());
             m_view->canvas()->update();
         } else {
             KMessageBox::error(m_view, i18n("Could not add layer to image."), i18n("Layer Error"));
@@ -521,8 +537,8 @@ void KisLayerManager::addCloneLayer(KisNodeSP parent, KisNodeSP above)
 
         if (layer) {
 
-            layer->setCompositeOp(img->colorSpace()->compositeOp(COMPOSITE_OVER));
-            img->addNode(layer.data(), parent.data(), above.data());
+            layer->setCompositeOp(COMPOSITE_OVER);
+            m_commandsAdapter->addNode(layer.data(), parent.data(), above.data());
 
             m_view->canvas()->update();
 
@@ -553,10 +569,10 @@ void KisLayerManager::addShapeLayer(KisNodeSP parent, KisNodeSP above)
             dynamic_cast<KoShapeContainer*>(m_doc->shapeForNode(parentLayer));
         if (!parentContainer) return;
 
-        KisLayerSP layer = new KisShapeLayer(parentContainer, img.data(), img->nextLayerName(), OPACITY_OPAQUE);
+        KisLayerSP layer = new KisShapeLayer(parentContainer, m_doc->shapeController(), img.data(), img->nextLayerName(), OPACITY_OPAQUE);
         if (layer) {
-            layer->setCompositeOp(img->colorSpace()->compositeOp(COMPOSITE_OVER));
-            img->addNode(layer.data(), parent, above.data());
+            layer->setCompositeOp(COMPOSITE_OVER);
+            m_commandsAdapter->addNode(layer.data(), parent, above.data());
             m_view->canvas()->update();
         } else {
             KMessageBox::error(m_view, i18n("Could not add layer to image."), i18n("Layer Error"));
@@ -590,6 +606,8 @@ void KisLayerManager::addAdjustmentLayer(KisNodeSP parent, KisNodeSP above)
     KisAdjustmentLayerSP adjl = addAdjustmentLayer(parent, above, QString(), 0, selection);
 
     KisDlgAdjustmentLayer dlg(adjl, adjl.data(), dev, adjl->image(), img->nextLayerName(), i18n("New Filter Layer"), m_view, "dlgadjustmentlayer");
+    dlg.resize( dlg.minimumSizeHint() );
+
     if (dlg.exec() != QDialog::Accepted) {
         m_view->image()->removeNode(adjl);
     } else {
@@ -606,7 +624,7 @@ KisAdjustmentLayerSP KisLayerManager::addAdjustmentLayer(KisNodeSP parent, KisNo
     if (!img) return 0;
 
     KisAdjustmentLayerSP l = new KisAdjustmentLayer(img, name, filter, selection);
-    img->addNode(l.data(), parent, above);
+    m_commandsAdapter->addNode(l.data(), parent, above);
     l->setDirty(img->bounds());
     return l;
 }
@@ -624,6 +642,8 @@ void KisLayerManager::addGeneratorLayer(KisNodeSP parent, KisNodeSP above)
     if (!img) return;
 
     KisDlgGeneratorLayer dlg(img->nextLayerName(), m_view);
+    dlg.resize( dlg.minimumSizeHint() );
+
     if (dlg.exec() == QDialog::Accepted) {
         KisSelectionSP selection = m_view->selection();
         KisFilterConfiguration * generator = dlg.configuration();
@@ -642,7 +662,7 @@ void KisLayerManager::addGeneratorLayer(KisNodeSP parent, KisNodeSP above, const
     if (!img) return;
 
     KisGeneratorLayerSP l = new KisGeneratorLayer(img, name, generator, selection);
-    img->addNode(l.data(), parent, above.data());
+    m_commandsAdapter->addNode(l.data(), parent, above.data());
     if (l->selection())
         l->setDirty(l->selection()->selectedExactRect());
     else
@@ -661,7 +681,7 @@ void KisLayerManager::layerRemove()
             QRect extent = layer->extent();
             KisNodeSP parent = layer->parent();
 
-            img->removeLayer(layer);
+            m_commandsAdapter->removeNode(layer);
 
             if (parent)
                 parent->setDirty(extent);
@@ -686,7 +706,7 @@ void KisLayerManager::layerDuplicate()
 
     KisLayerSP dup = dynamic_cast<KisLayer*>(active->clone().data());
     dup->setName(i18n("Duplicate of '%1'", active->name()));
-    img->addNode(dup.data(), active->parent(), active.data());
+    m_commandsAdapter->addNode(dup.data(), active->parent(), active.data());
     if (dup) {
         activateLayer(dup);
         dup->setDirty();
@@ -706,7 +726,7 @@ void KisLayerManager::layerRaise()
 
     layer = activeLayer();
 
-    img->raiseLayer(layer);
+    m_commandsAdapter->raise(layer);
     layer->parent()->setDirty();
 }
 
@@ -720,7 +740,7 @@ void KisLayerManager::layerLower()
 
     layer = activeLayer();
 
-    img->lowerLayer(layer);
+    m_commandsAdapter->lower( layer );
     layer->parent()->setDirty();
 }
 
@@ -733,7 +753,7 @@ void KisLayerManager::layerFront()
         return;
 
     layer = activeLayer();
-    img->toTop(layer.data());
+    m_commandsAdapter->toTop(layer);
     layer->parent()->setDirty();
 }
 
@@ -745,7 +765,7 @@ void KisLayerManager::layerBack()
     KisLayerSP layer;
 
     layer = activeLayer();
-    img->toBottom(layer.data());
+    m_commandsAdapter->toBottom(layer);
     layer->parent()->setDirty();
 }
 
@@ -777,7 +797,7 @@ void KisLayerManager::mirrorLayerX()
 
     QRect dirty = KisTransformWorker::mirrorX(dev, m_view->selection() );
     m_activeLayer->setDirty(dirty);
-    
+
     if (t) m_view->undoAdapter()->addCommand(t);
 
     m_doc->setModified(true);
@@ -931,7 +951,7 @@ void KisLayerManager::mergeLayer()
     if(newLayer) {
         newLayer->setDirty();
     }
-    
+
     m_view->updateGUI();
 
 }
