@@ -860,6 +860,11 @@ bool Cell::isTime() const
     return (Format::isTime(t) || ((t == Format::Generic) && (value().format() == Value::fmt_Time)));
 }
 
+bool Cell::isText() const
+{
+    const Format::Type t = style().formatType();
+    return t == Format::Text;
+}
 
 // Return true if this cell is part of a merged cell, but not the
 // master cell.
@@ -1130,7 +1135,7 @@ bool Cell::saveOdf( KoXmlWriter& xmlwriter, KoGenStyles &mainStyles,
     // group empty cells with the same style
     const QString comment = this->comment();
     if (isEmpty() && comment.isEmpty() && !isPartOfMerged() && !doesMergeCells() &&
-        !tableContext.cellHasAnchoredShapes(*this)) {
+        !tableContext.cellHasAnchoredShapes(sheet(), row, column)) {
       bool refCellIsDefault = isDefault();
       int j = column + 1;
       Cell nextCell = sheet()->cellStorage()->nextInRow( column, row );
@@ -1140,7 +1145,7 @@ bool Cell::saveOdf( KoXmlWriter& xmlwriter, KoGenStyles &mainStyles,
         //   the next cell is not the adjacent one
         // or
         //   the next cell is not empty
-        if (nextCell.column() != j || (!nextCell.isEmpty() || tableContext.cellHasAnchoredShapes(*this))) {
+        if (nextCell.column() != j || (!nextCell.isEmpty() || tableContext.cellHasAnchoredShapes(sheet(), row, column))) {
           if ( refCellIsDefault )
           {
             // if the origin cell was a default cell,
@@ -1154,7 +1159,7 @@ bool Cell::saveOdf( KoXmlWriter& xmlwriter, KoGenStyles &mainStyles,
         }
 
         if ( nextCell.isPartOfMerged() || nextCell.doesMergeCells() ||
-             !nextCell.comment().isEmpty() || tableContext.cellHasAnchoredShapes(*this) ||
+             !nextCell.comment().isEmpty() || tableContext.cellHasAnchoredShapes(sheet(), row, column) ||
              !(nextCell.style() == style() && nextCell.conditions() == conditions()) )
         {
           break;
@@ -1221,8 +1226,8 @@ bool Cell::saveOdf( KoXmlWriter& xmlwriter, KoGenStyles &mainStyles,
     // Save shapes that are anchored to this cell.
     // see: OpenDocument, 2.3.1 Text Documents
     // see: OpenDocument, 9.2 Drawing Shapes
-    if (tableContext.cellHasAnchoredShapes(*this)) {
-        const QList<KoShape*> shapes = tableContext.cellAnchoredShapes(*this);
+    if (tableContext.cellHasAnchoredShapes(sheet(), row, column)) {
+        const QList<KoShape*> shapes = tableContext.cellAnchoredShapes(sheet(), row, column);
         for (int i = 0; i < shapes.count(); ++i) {
             KoShape* const shape = shapes[i];
             const QPointF bottomRight = shape->boundingRect().bottomRight();
@@ -1260,11 +1265,23 @@ void Cell::saveOdfValue (KoXmlWriter &xmlWriter)
     }
     case Value::fmt_Number:
     {
-      xmlWriter.addAttribute( "office:value-type", "float" );
-      if (value().isInteger())
-        xmlWriter.addAttribute( "office:value", QString::number( value().asInteger() ) );
-      else
-        xmlWriter.addAttribute( "office:value", QString::number( numToDouble (value().asFloat()), 'g', DBL_DIG ) );
+      if (isDate()) {
+        xmlWriter.addAttribute( "office:value-type", "date" );
+        xmlWriter.addAttribute( "office:date-value",
+            value().asDate(sheet()->map()->calculationSettings()).toString( Qt::ISODate ) );
+      } else if (isText()) {
+        xmlWriter.addAttribute( "office:value-type", "string" );
+        if (value().isInteger())
+          xmlWriter.addAttribute( "office:string-value", QString::number( value().asInteger() ) );
+        else
+          xmlWriter.addAttribute( "office:string-value", QString::number( numToDouble (value().asFloat()), 'g', DBL_DIG ) );
+      } else {
+        xmlWriter.addAttribute( "office:value-type", "float" );
+        if (value().isInteger())
+          xmlWriter.addAttribute( "office:value", QString::number( value().asInteger() ) );
+        else
+          xmlWriter.addAttribute( "office:value", QString::number( numToDouble (value().asFloat()), 'g', DBL_DIG ) );
+      }
       break;
     }
     case Value::fmt_Percent:
@@ -1289,9 +1306,15 @@ void Cell::saveOdfValue (KoXmlWriter &xmlWriter)
     case Value::fmt_DateTime: break;  //NOTHING HERE
     case Value::fmt_Date:
     {
-      xmlWriter.addAttribute( "office:value-type", "date" );
-      xmlWriter.addAttribute( "office:date-value",
-          value().asDate(sheet()->map()->calculationSettings()).toString( Qt::ISODate ) );
+      if (isTime()) {
+        xmlWriter.addAttribute( "office:value-type", "time" );
+        xmlWriter.addAttribute( "office:time-value",
+            value().asTime(sheet()->map()->calculationSettings()).toString( "PThhHmmMssS" ) );
+      } else {
+        xmlWriter.addAttribute( "office:value-type", "date" );
+        xmlWriter.addAttribute( "office:date-value",
+            value().asDate(sheet()->map()->calculationSettings()).toString( Qt::ISODate ) );
+      }
       break;
     }
     case Value::fmt_Time:
