@@ -44,6 +44,8 @@
 
 // koffice libs includes
 #include <KoShapeManager.h>
+#include <KoTextDocument.h>
+#include <KoTextAnchor.h>
 #include <KoShapeContainer.h>
 #include <KoOdfWriteStore.h>
 #include <KoToolManager.h>
@@ -72,12 +74,63 @@
 #include <QCoreApplication>
 #include <QTextBlock>
 
+/// \internal
+// this class will be added to all views and be hidden by default.
+// during loading any frames we find will be added here and only when they are positioned properly will
+// those frames be re-shown.
+class MagicCurtain : public KoShapeContainer
+{
+public:
+    // reimplemented pure virtual calls
+    bool loadOdf(const KoXmlElement&, KoShapeLoadingContext&) { return false; }
+    void saveOdf(KoShapeSavingContext&) const { }
+    void paintComponent(QPainter&, const KoViewConverter&) { }
+
+    /// add the frame to be hidden
+    void addFrame(KWFrame *frame);
+    /// add the shape to be hidden
+    void addShape(KoShape *shape);
+    // reveal all the frames that were added before
+    void revealFramesForPage(int pageNumber, qreal moveFrames);
+
+private:
+    QHash<int, QList<KWFrame*> > m_data;
+};
+
+void MagicCurtain::addFrame(KWFrame *frame)
+{
+    Q_ASSERT(frame->loadingPageNumber() > 0);
+    QList<KWFrame*> frames = m_data.value(frame->loadingPageNumber());
+    frames << frame;
+    m_data.insert(frame->loadingPageNumber(), frames);
+    frame->shape()->setParent(this);
+}
+
+void MagicCurtain::addShape(KoShape *shape)
+{
+    shape->setParent(this);
+}
+
+void MagicCurtain::revealFramesForPage(int pageNumber, qreal moveFrames)
+{
+    QPointF offset(0, moveFrames);
+    foreach (KWFrame *frame, m_data.value(pageNumber)) {
+        frame->shape()->setPosition(frame->shape()->position() + offset);
+        frame->shape()->setParent(0);
+        frame->clearLoadingData();
+    }
+    m_data.remove(pageNumber);
+}
+
+
+// KWDocument
 KWDocument::KWDocument(QWidget *parentWidget, QObject* parent, bool singleViewMode)
         : KoDocument(parentWidget, parent, singleViewMode),
-        m_frameLayout(&m_pageManager, m_frameSets)
+        m_frameLayout(&m_pageManager, m_frameSets),
+        m_magicCurtain(0),
+        m_mainFramesetEverFinished(false)
 {
     m_frameLayout.setDocument(this);
-    m_inlineTextObjectManager = new KoInlineTextObjectManager(this);
 
     connect(documentInfo(), SIGNAL(infoUpdated(const QString &, const QString &)),
             inlineTextObjectManager(), SLOT(documentInformationUpdated(const QString &, const QString &)));
@@ -102,11 +155,10 @@ KWDocument::KWDocument(QWidget *parentWidget, QObject* parent, bool singleViewMo
 
 KWDocument::~KWDocument()
 {
+    delete m_magicCurtain;
     saveConfig();
     qDeleteAll(m_frameSets);
     qDeleteAll(m_dataCenterMap);
-    delete m_inlineTextObjectManager;
-    m_inlineTextObjectManager = 0;
 }
 
 void KWDocument::addShape(KoShape *shape)
@@ -155,6 +207,8 @@ void KWDocument::paintContent(QPainter&, const QRect& rect)
 KoView* KWDocument::createViewInstance(QWidget* parent)
 {
     KWView *view = new KWView(m_viewMode, this, parent);
+    if (m_magicCurtain)
+        view->kwcanvas()->shapeManager()->add(m_magicCurtain);
     bool switchToolCalled = false;
     foreach (KWFrameSet *fs, m_frameSets) {
         if (fs->frameCount() == 0)
@@ -211,7 +265,7 @@ void KWDocument::removePage(int pageNumber)
 
 void KWDocument::firePageSetupChanged()
 {
-    m_inlineTextObjectManager->setProperty(KoInlineObject::PageCount, pageCount());
+    inlineTextObjectManager()->setProperty(KoInlineObject::PageCount, pageCount());
     emit pageSetupChanged();
 }
 
@@ -275,6 +329,11 @@ void KWDocument::addFrameSet(KWFrameSet *fs)
                 tfs->textFrameSetType() == KWord::OtherTextFrameSet) {
             connect(tfs, SIGNAL(moreFramesNeeded(KWTextFrameSet*)),
                     this, SLOT(requestMoreSpace(KWTextFrameSet*)));
+            connect(tfs, SIGNAL(layoutDone()), this, SLOT(mainTextFrameSetLayoutDone()));
+        }
+        else {
+            connect(tfs, SIGNAL(decorationFrameResize(KWTextFrameSet*)),
+                    this, SLOT(updateHeaderFooter(KWTextFrameSet*)));
         }
     }
 
@@ -292,7 +351,17 @@ void KWDocument::addFrame(KWFrame *frame)
             canvas->shapeManager()->add(frame->shape());
         canvas->resourceProvider()->setResource(KWord::CurrentFrameSetCount, m_frameSets.count());
     }
-    frame->shape()->update();
+    if (frame->loadingPageNumber() > 0) {
+        if (m_magicCurtain == 0) {
+            m_magicCurtain = new MagicCurtain();
+            m_magicCurtain->setVisible(false);
+            foreach (KoView *view, views())
+                static_cast<KWView*>(view)->kwcanvas()->shapeManager()->add(m_magicCurtain);
+        }
+        m_magicCurtain->addFrame(frame);
+    }
+    else
+        frame->shape()->update();
 }
 
 void KWDocument::removeFrame(KWFrame *frame)
@@ -315,6 +384,11 @@ void KWDocument::removeFrame(KWFrame *frame)
     delete cmd;
 }
 
+void KWDocument::mainTextFrameSetLayoutDone()
+{
+    m_mainFramesetEverFinished = true;
+}
+
 KWFrameSet *KWDocument::frameSetByName(const QString & name)
 {
     foreach (KWFrameSet *fs, m_frameSets) {
@@ -329,6 +403,10 @@ KWTextFrameSet *KWDocument::mainFrameSet() const
     return m_frameLayout.mainFrameSet();
 }
 
+KoInlineTextObjectManager *KWDocument::inlineTextObjectManager() const
+{
+    return dynamic_cast<KoInlineTextObjectManager*>(dataCenterMap()["InlineTextObjectManager"]);
+}
 
 QString KWDocument::uniqueFrameSetName(const QString& suggestion)
 {
@@ -439,7 +517,7 @@ void KWDocument::clear()
     padding.right = MM_TO_POINT(3);
     m_pageManager.setPadding(padding);
 
-    m_inlineTextObjectManager->setProperty(KoInlineObject::PageCount, pageCount());
+    inlineTextObjectManager()->setProperty(KoInlineObject::PageCount, pageCount());
 }
 
 bool KWDocument::loadOdf(KoOdfReadStore & odfStore)
@@ -474,8 +552,11 @@ bool KWDocument::loadXML(const KoXmlDocument & doc, KoStore *store)
 void KWDocument::endOfLoading() // called by both oasis and oldxml
 {
     // Get the master page name of the first page.
-    QTextBlock block = mainFrameSet()->document()->firstBlock();
-    QString firstPageMasterName = block.blockFormat().stringProperty(KoParagraphStyle::MasterPageName);
+    QString firstPageMasterName;
+    if (mainFrameSet()) {
+        QTextBlock block = mainFrameSet()->document()->firstBlock();
+        firstPageMasterName = block.blockFormat().stringProperty(KoParagraphStyle::MasterPageName);
+    }
 
     KWPage lastpage = pageManager()->last();
     qreal docHeight = lastpage.isValid() ? (lastpage.offsetInDocument() + lastpage.height()) : 0.0;
@@ -487,18 +568,22 @@ void KWDocument::endOfLoading() // called by both oasis and oldxml
         foreach (KWFrame *frame, fs->frames())
         maxBottom = qMax(maxBottom, frame->shape()->boundingRect().bottom());
     }
-    // My dear reader, please listen carefully
-    // Else your failure will be miserably
-    // Don't trust what your eyes see
-    // This while you should not see
-    // Only one page is created
-    // 'Cause layout is not finished
-    // My dear reader, please listen carefully
-    // Else your failure will be miserably
-    // Don't trust what your eyes see
-    // This while you should not see
-    // Only one page is created
-    // 'Cause layout is not finished
+    // The Document we loaded could have specified
+    //  1) a number of pages
+    //  2) a number of frames
+    // At the end of loading we then end up in one of 3 situations.
+    // a) we have exactly the amount of pages that the document needs.
+    // b) we have absolute frames positioned on pages that don't exist.
+    // c) we have so much text in any of our text-framesets that new pages
+    //    may have to be generated at some time after loading is completed.
+
+    if (m_magicCurtain) { // pages defined in the loaded-document
+        foreach (KWPage page, m_pageManager.pages())
+            m_magicCurtain->revealFramesForPage(page.pageNumber(), page.offsetInDocument());
+    }
+
+    // Here we look at point 'b'. We add pages so at least all frames have a page.
+    // btw. the observent reader might notice that cases b and c are not mutually exclusive ;)
     while (docHeight <= maxBottom) {
         kDebug(32001) << "KWDocument::endOfLoading appends a page";
         if (m_pageManager.pageCount() == 0) // apply the firstPageMasterName only on the first page
@@ -507,6 +592,9 @@ void KWDocument::endOfLoading() // called by both oasis and oldxml
             lastpage = m_pageManager.appendPage();
         ppq->addPage(lastpage);
         docHeight += lastpage.height();
+        if (m_magicCurtain) {
+            m_magicCurtain->revealFramesForPage(lastpage.pageNumber(), lastpage.offsetInDocument());
+        }
     }
 
 #if 0
@@ -584,26 +672,29 @@ void KWDocument::endOfLoading() // called by both oasis and oldxml
 
     foreach (KWFrameSet *fs, m_frameSets) {
         KWTextFrameSet *tfs = dynamic_cast<KWTextFrameSet*>(fs);
-        if (tfs)
-            tfs->setAllowLayout(true);
+        if (!tfs)
+            continue;
+        KoTextDocument textDoc(tfs->document());
+        foreach (KoInlineObject *inlineObject, textDoc.inlineTextObjectManager()->inlineTextObjects()) {
+            KoTextAnchor *anchor = dynamic_cast<KoTextAnchor*>(inlineObject);
+            if (anchor) {
+                if (m_magicCurtain == 0) {
+                    m_magicCurtain = new MagicCurtain();
+                    m_magicCurtain->setVisible(false);
+                    foreach (KoView *view, views())
+                        static_cast<KWView*>(view)->kwcanvas()->shapeManager()->add(m_magicCurtain);
+                }
+                m_magicCurtain->addShape(anchor->shape());
+            }
+        }
+        tfs->setAllowLayout(true);
     }
 
     emit sigProgress(-1);
 
     kDebug(32001) << "KWDocument::endOfLoading done";
 
-#if 0
-    // Connect to notifications from main text-frameset
-    if (frameset) {
-        connect(frameset->textObject(), SIGNAL(chapterParagraphFormatted(KoTextParag *)),
-                SLOT(slotChapterParagraphFormatted(KoTextParag *)));
-        connect(frameset, SIGNAL(mainTextHeightChanged()),
-                SIGNAL(mainTextHeightChanged()));
-    }
-#endif
-
     // Note that more stuff will happen in completeLoading
-
     firePageSetupChanged();
     setModified(false);
 }
@@ -629,7 +720,7 @@ bool KWDocument::saveOdf(SavingContext &documentContext)
 
 void KWDocument::requestMoreSpace(KWTextFrameSet *fs)
 {
-    //kDebug(32002) <<"KWDocument::requestMoreSpace";
+    // kDebug(32002) << fs;
     Q_ASSERT(fs);
     Q_ASSERT(fs->frameCount() > 0);
     Q_ASSERT(QThread::currentThread() == thread());
@@ -658,8 +749,26 @@ void KWDocument::requestMoreSpace(KWTextFrameSet *fs)
         // its enough to just create a new frame.
         m_frameLayout.createNewFrameForPage(fs, page.pageNumber() +
                                             (lastFrame->frameOnBothSheets() ? 1 : 2));
-    } else
-        appendPage(masterPageName);
+    } else {
+        KWPage newPage = appendPage(masterPageName);
+        if (m_magicCurtain)
+            m_magicCurtain->revealFramesForPage(newPage.pageNumber(), newPage.offsetInDocument());
+    }
+}
+
+void KWDocument::updateHeaderFooter(KWTextFrameSet *tfs)
+{
+    // find all pages that have the page style set and re-layout them.
+    Q_ASSERT(tfs->pageStyle().isValid());
+    PageProcessingQueue *ppq = 0;
+    const KWPageStyle style = tfs->pageStyle();
+    foreach (KWPage page, pageManager()->pages()) {
+        if (page.pageStyle() == style) {
+            if (ppq == 0)
+                ppq = new PageProcessingQueue(this);
+            ppq->addPage(page);
+        }
+    }
 }
 
 void KWDocument::showStartUpWidget(KoMainWindow* parent, bool alwaysShow)
@@ -791,6 +900,8 @@ void PageProcessingQueue::process()
     const bool docIsEmpty = m_document->isEmpty();
     const bool docIsModified = m_document->isModified();
     foreach (const KWPage &page, m_pages) {
+        if (! page.isValid())
+            continue;
         emit m_document->pageSetupChanged();
         m_document->m_frameLayout.createNewFramesForPage(page.pageNumber());
     }
