@@ -1,6 +1,6 @@
 /* This file is part of the KDE project
  * Copyright (C) 2001 David Faure <faure@kde.org>
- * Copyright (C) 2005-2007 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2005-2007, 2009 Thomas Zander <zander@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -63,10 +63,12 @@
 #include <KoZoomController.h>
 #include <KoInlineTextObjectManager.h>
 #include <KoBookmark.h>
+#include <KoPathShape.h> // for KoPathShapeId
 
 // KDE + Qt includes
 #include <QHBoxLayout>
 #include <QMenu>
+#include <QTimer>
 #include <klocale.h>
 #include <kdebug.h>
 #include <kicon.h>
@@ -92,6 +94,7 @@ KWView::KWView(const QString& viewMode, KWDocument* document, QWidget *parent)
     m_snapToGrid = m_document->gridData().snapToGrid();
     m_gui = new KWGui(viewMode, this);
     m_canvas = m_gui->canvas();
+    setFocusProxy(m_canvas);
 
     QHBoxLayout *layout = new QHBoxLayout(this);
     layout->setMargin(0);
@@ -109,6 +112,16 @@ KWView::KWView(const QString& viewMode, KWDocument* document, QWidget *parent)
     new KoFind(this, m_canvas->resourceProvider(), actionCollection());
 
     m_zoomController = new KoZoomController(m_gui->canvasController(), &m_zoomHandler, actionCollection());
+
+    KWStatisticsDockerFactory statisticsFactory(this);
+    KWStatisticsDocker *docker = dynamic_cast<KWStatisticsDocker *>(createDockWidget(&statisticsFactory));
+    if (docker && docker->view() != this) docker->setView(this);
+
+    if (statusBar())
+        new KWStatusBar(statusBar(), this);
+
+    // the zoom controller needs to be initialized after the status bar gets initialized as 
+    // that resulted in bug 180759
     m_zoomController->setPageSize(m_currentPage.rect().size());
     KoZoomMode::Modes modes = KoZoomMode::ZOOM_WIDTH;
     if (m_canvas->viewMode()->hasPages())
@@ -118,12 +131,6 @@ KWView::KWView(const QString& viewMode, KWDocument* document, QWidget *parent)
     m_canvas->updateSize(); // to emit the doc size at least once
     m_zoomController->setZoom(m_document->config().zoomMode(), m_document->config().zoom() / 100.);
     connect(m_zoomController, SIGNAL(zoomChanged(KoZoomMode::Mode, qreal)), this, SLOT(zoomChanged(KoZoomMode::Mode, qreal)));
-
-    KWStatisticsDockerFactory statisticsFactory(this);
-    KWStatisticsDocker *docker = dynamic_cast<KWStatisticsDocker *>(createDockWidget(&statisticsFactory));
-    if (docker && docker->view() != this) docker->setView(this);
-
-    new KWStatusBar(statusBar(), this);
 }
 
 KWView::~KWView()
@@ -275,17 +282,19 @@ void KWView::setupActions()
     connect(action, SIGNAL(toggled(bool)), this, SLOT(showRulers(bool)));
 
     action = new KAction(i18n("Delete Page"), this);
-    actionCollection()->addAction("delete_page", action);
+    //actionCollection()->addAction("delete_page", action); // TODO reinstate this action when the command is able to delete something that is not the last page
     connect(action, SIGNAL(triggered()), this, SLOT(deletePage()));
     handleDeletePageAction(); //decide if we enable or disable this action
     connect(m_document, SIGNAL(pageSetupChanged()), this, SLOT(handleDeletePageAction()));
 
+if (false) { // TODO move this to the text tool as soon as  a) the string freeze is lifted.  b) Qt45 makes this possible
     action = new KAction(i18n("Formatting Characters"), this);
     action->setCheckable(true);
     actionCollection()->addAction("view_formattingchars", action);
     connect(action, SIGNAL(toggled(bool)), this, SLOT(setShowFormattingChars(bool)));
     action->setToolTip(i18n("Toggle the display of non-printing characters"));
     action->setWhatsThis(i18n("Toggle the display of non-printing characters.<br/><br/>When this is enabled, KWord shows you tabs, spaces, carriage returns and other non-printing characters."));
+}
 
     action = new KAction(i18n("Select All Frames"), this);
 
@@ -298,10 +307,10 @@ void KWView::setupActions()
     connect(kwcanvas()->toolProxy(), SIGNAL(selectionChanged(bool)), action, SLOT(setEnabled(bool)));
     actionCollection()->addAction("edit_delete", action );
 
-    action = new KAction(i18n("Show Grid"), this);
-    action->setCheckable(true);
+    action = m_document->gridData().gridToggleAction(m_canvas);
+    // XXX remove the translated string when the string freeze is lifted, the KoGridData should have those
+    action->setText(i18n("Show Grid"));
     actionCollection()->addAction("view_grid", action);
-    connect(action, SIGNAL(toggled(bool)), this, SLOT(viewGrid(bool)));
 
     // -------------- Frame menu
     action  = new KAction(i18n("Create Linked Copy"), this);
@@ -970,6 +979,7 @@ void KWView::toggleSnapToGrid()
 
 void KWView::adjustZOrderOfSelectedFrames(KoShapeReorderCommand::MoveShapeType direction)
 {
+    // TODO we should not allow any shapes to fall behind the main text frame.
     QUndoCommand *cmd = KoShapeReorderCommand::createCommand(kwcanvas()->shapeManager()->selection()->selectedShapes(),
                         kwcanvas()->shapeManager(), direction);
     if (cmd)
@@ -1012,7 +1022,6 @@ void KWView::inlineFrame()
         return;
     }
 
-    KoTextAnchor *anchor = new KoTextAnchor(targetShape);
     selection->deselectAll();
     KWFrame *frameForAnchor = 0;
     int area = 0;
@@ -1030,7 +1039,7 @@ void KWView::inlineFrame()
     }
 
     if (frameForAnchor == 0) {/* can't happen later on... */
-        kDebug() << "splitting..."; return;
+        kDebug(32001) << "bailing out..."; return;
     }
 
     selection->select(frameForAnchor->shape());
@@ -1043,6 +1052,8 @@ void KWView::inlineFrame()
     KoToolManager::instance()->switchToolRequested(tool);
     KoTextSelectionHandler *handler = qobject_cast<KoTextSelectionHandler*> (kwcanvas()->toolProxy()->selection());
     Q_ASSERT(handler);
+    KoTextAnchor *anchor = new KoTextAnchor(targetShape);
+    anchor->setOffset(QPointF(0, -targetShape->size().height()));
     // TODO move caret
     handler->insertInlineObject(anchor);
 }
@@ -1131,11 +1142,6 @@ void KWView::editDeleteSelection()
     kwcanvas()->toolProxy()->deleteSelection();
 }
 
-void KWView::viewGrid(bool on)
-{
-    m_document->gridData().setShowGrid(on);
-}
-
 void KWView::createCustomOutline()
 {
     QList<KWFrame *> frames = selectedFrames();
@@ -1143,12 +1149,22 @@ void KWView::createCustomOutline()
         return;
     if (frames.count() == 1) {
         m_document->addCommand(new KWCreateOutlineCommand(m_document, frames.at(0)));
-        return;
+    } else {
+        QUndoCommand *cmd = new QUndoCommand(i18n("Create outlines"));
+        foreach (KWFrame *frame, frames)
+            new KWCreateOutlineCommand(m_document, frame, cmd);
+        m_document->addCommand(cmd);
     }
-    QUndoCommand *cmd = new QUndoCommand(i18n("Create outlines"));
-    foreach (KWFrame *frame, frames)
-        new KWCreateOutlineCommand(m_document, frame, cmd);
-    m_document->addCommand(cmd);
+
+    KoSelection *selection = kwcanvas()->shapeManager()->selection();
+    selection->deselectAll();
+    foreach (KWFrame *frame, frames) {
+        KoShapeContainer *group = frame->shape()->parent();
+        if (group)
+            selection->select(group);
+    }
+
+    KoToolManager::instance()->switchToolRequested("PathToolFactoryId");
 }
 
 
@@ -1196,15 +1212,24 @@ void KWView::selectionChanged()
     foreach (KoShape *shape, kwcanvas()->shapeManager()->selection()->selectedShapes(KoFlake::TopLevelSelection)) {
         KWFrame *frame = frameForShape(shape);
         Q_ASSERT(frame);
-        m_canvas->resourceProvider()->setResource(KWord::CurrentFrame, frame);
-        m_canvas->resourceProvider()->setResource(KWord::CurrentFrameSet, frame->frameSet());
+        QVariant variant;
+        variant.setValue<void*>(frame);
+        m_canvas->resourceProvider()->setResource(KWord::CurrentFrame, variant);
+        variant.setValue<void*>(frame->frameSet());
+        m_canvas->resourceProvider()->setResource(KWord::CurrentFrameSet, variant);
         break;
     }
 }
 
-void KWView::sanityCheck()
+void KWView::showEvent(QShowEvent *e)
+{
+    KoView::showEvent(e);
+    QTimer::singleShot(0, this, SLOT(updateStatusBarAction()));
+}
+
+void KWView::updateStatusBarAction()
 {
     KToggleAction *action = (KToggleAction*) actionCollection()->action("showStatusBar");
-    if (action)
-        action->setChecked(statusBar()->isVisible());
+    if (action && statusBar())
+        action->setChecked(! statusBar()->isHidden());
 }
