@@ -30,6 +30,7 @@
 #include <KoInsets.h>
 #include <KoShapeManager.h>
 #include <kdeversion.h>
+#include <QApplication>
 
 KWPrintingDialog::KWPrintingDialog(KWView *view)
         : KoPrintingDialog(view),
@@ -37,15 +38,20 @@ KWPrintingDialog::KWPrintingDialog(KWView *view)
         m_clipToPage(false)
 {
     setShapeManager(view->kwcanvas()->shapeManager());
+
+    while (! m_document->layoutFinishedAtleastOnce()) {
+        QCoreApplication::processEvents();
+        if (! QCoreApplication::hasPendingEvents())
+            break;
+    }
     printer().setFromTo(documentFirstPage(), documentLastPage());
-    // TODO if the doc is not yet done layouting, the lastpage is not correct
 }
 
 KWPrintingDialog::~KWPrintingDialog()
 {
 }
 
-void KWPrintingDialog::preparePage(int pageNumber)
+QRectF KWPrintingDialog::preparePage(int pageNumber)
 {
     const int resolution = printer().resolution();
     KoInsets bleed = m_document->pageManager()->padding();
@@ -55,7 +61,7 @@ void KWPrintingDialog::preparePage(int pageNumber)
 
     KWPage page = m_document->pageManager()->page(pageNumber);
     if (! page.isValid())
-        return;
+        return QRectF();
     QRectF pageRect = page.rect(pageNumber);
     pageRect.adjust(-bleed.left, -bleed.top, bleed.right, bleed.bottom);
 
@@ -94,8 +100,7 @@ void KWPrintingDialog::preparePage(int pageNumber)
         }
     }
 
-    m_currentPage = QRectF(offset, pageOffset, clipWidth + bleedWidth, clipHeight + bleedHeigt);
-    painter().setClipRect(m_currentPage);
+    return QRectF(offset, pageOffset, clipWidth + bleedWidth, clipHeight + bleedHeigt);
 }
 
 QList<KoShape*> KWPrintingDialog::shapesOnPage(int pageNumber)
