@@ -1,7 +1,7 @@
 /* This file is part of the KDE project
  * Copyright (C) 2007 Boudewijn Rempt <boud@kde.org>
- * Copyright (C) 2007 Thorsten Zachmann <zachmann@kde.org>
- * Copyright (C) 2007 Jan Hambrecht <jaham@gmx.net>
+ * Copyright (C) 2007,2009 Thorsten Zachmann <zachmann@kde.org>
+ * Copyright (C) 2007,2009  Jan Hambrecht <jaham@gmx.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -29,6 +29,7 @@
 #include <KoXmlNS.h>
 #include <KoStoreDevice.h>
 #include <KoUnit.h>
+#include "KoConnectionShapeLoadingUpdater.h"
 
 #include <kdebug.h>
 
@@ -83,12 +84,52 @@ void KoConnectionShape::paint(QPainter&, const KoViewConverter&)
 
 void KoConnectionShape::saveOdf(KoShapeSavingContext & context) const
 {
-    Q_UNUSED(context);
+    context.xmlWriter().startElement("draw:connector");
+    saveOdfAttributes( context, OdfMandatories | OdfAdditionalAttributes );
+    
+    switch( d->connectionType )
+    {
+        case Lines:
+            context.xmlWriter().addAttribute( "draw:type", "lines" );
+            break;
+        case Straight:
+            context.xmlWriter().addAttribute( "draw:type", "line" );
+            break;
+        case Curve:
+            context.xmlWriter().addAttribute( "draw:type", "curve" );
+            break;
+        default:
+            context.xmlWriter().addAttribute( "draw:type", "standard" );
+            break;
+    }
+    
+    if (d->shape1) {
+        context.xmlWriter().addAttribute( "draw:start-shape", context.drawId(d->shape1) );
+        context.xmlWriter().addAttribute( "draw:start-glue-point", d->connectionPointIndex1 );
+    }
+    else {
+        QPointF p((m_handles[0] + position()) * context.shapeOffset(this));
+        context.xmlWriter().addAttributePt( "svg:x1", p.x() );
+        context.xmlWriter().addAttributePt( "svg:y1", p.y() );
+    }
+    if (d->shape2) {
+        context.xmlWriter().addAttribute( "draw:end-shape", context.drawId(d->shape2) );
+        context.xmlWriter().addAttribute( "draw:end-glue-point", d->connectionPointIndex2 );
+    }
+    else {
+        QPointF p((m_handles[1] + position()) * context.shapeOffset(this));
+        context.xmlWriter().addAttributePt( "svg:x2", p.x() );
+        context.xmlWriter().addAttributePt( "svg:y2", p.y() );
+    }
+
+    saveOdfCommonChildElements(context);
+    
+    context.xmlWriter().endElement();
 }
 
 bool KoConnectionShape::loadOdf(const KoXmlElement & element, KoShapeLoadingContext &context)
 {
-    loadOdfAttributes(element, context, OdfMandatories | OdfCommonChildElements);
+    loadOdfAttributes(element, context, OdfMandatories | OdfCommonChildElements | OdfAdditionalAttributes);
 
     QString type = element.attributeNS(KoXmlNS::draw, "type", "standard");
     if (type == "lines")
@@ -101,22 +142,30 @@ bool KoConnectionShape::loadOdf(const KoXmlElement & element, KoShapeLoadingCont
         d->connectionType = Standard;
 
     if (element.hasAttributeNS(KoXmlNS::draw, "start-shape")) {
+        d->connectionPointIndex1 = element.attributeNS(KoXmlNS::draw, "start-glue-point", "").toInt();
         QString shapeId1 = element.attributeNS(KoXmlNS::draw, "start-shape", "");
         d->shape1 = context.shapeById(shapeId1);
-        if (d->shape1)
+        if (d->shape1) {
             d->shape1->addDependee(this);
-        d->connectionPointIndex1 = element.attributeNS(KoXmlNS::draw, "start-glue-point", "").toInt();
+        }
+        else {
+            context.updateShape(shapeId1, new KoConnectionShapeLoadingUpdater(this, KoConnectionShapeLoadingUpdater::First));
+        }
     } else {
         m_handles[0].setX(KoUnit::parseValue(element.attributeNS(KoXmlNS::svg, "x1", QString())));
         m_handles[0].setY(KoUnit::parseValue(element.attributeNS(KoXmlNS::svg, "y1", QString())));
     }
 
     if (element.hasAttributeNS(KoXmlNS::draw, "end-shape")) {
+        d->connectionPointIndex2 = element.attributeNS(KoXmlNS::draw, "end-glue-point", "").toInt();
         QString shapeId2 = element.attributeNS(KoXmlNS::draw, "end-shape", "");
         d->shape2 = context.shapeById(shapeId2);
-        if (d->shape2)
+        if (d->shape2) {
             d->shape2->addDependee(this);
-        d->connectionPointIndex2 = element.attributeNS(KoXmlNS::draw, "end-glue-point", "").toInt();
+        }
+        else {
+            context.updateShape(shapeId2, new KoConnectionShapeLoadingUpdater(this, KoConnectionShapeLoadingUpdater::Second));
+        }
     } else {
         m_handles[1].setX(KoUnit::parseValue(element.attributeNS(KoXmlNS::svg, "x2", QString())));
         m_handles[1].setY(KoUnit::parseValue(element.attributeNS(KoXmlNS::svg, "y2", QString())));
@@ -131,6 +180,11 @@ bool KoConnectionShape::loadOdf(const KoXmlElement & element, KoShapeLoadingCont
     return true;
 }
 
+bool KoConnectionShape::hitTest(const QPointF &position) const
+{
+    return KoShape::hitTest(position);
+}
+
 void KoConnectionShape::moveHandleAction(int handleId, const QPointF & point, Qt::KeyboardModifiers modifiers)
 {
     Q_UNUSED(modifiers);
