@@ -40,12 +40,12 @@ KoEventActionFactory::~KoEventActionFactory()
     delete d;
 }
 
-QString KoEventActionFactory::action()
+const QString & KoEventActionFactory::action()
 {
     return d->action;
 }
 
-QString KoEventActionFactory::id() const
+const QString & KoEventActionFactory::id() const
 {
     return d->id;
 }
