@@ -1,6 +1,6 @@
 /* This file is part of the KDE project
    Copyright (C) 2006 Casper Boemann Rasmussen <cbr@boemann.dk>
-   Copyright (C) 2006-2007 Thomas Zander <zander@kde.org>
+   Copyright (C) 2006-2009 Thomas Zander <zander@kde.org>
    Copyright (C) 2006-2008 Thorsten Zachmann <zachmann@kde.org>
    Copyright (C) 2007 Jan Hambrecht <jaham@gmx.net>
 
@@ -43,6 +43,7 @@
 #include "KoShapeShadow.h"
 #include "KoEventAction.h"
 #include "KoEventActionRegistry.h"
+#include "KoOdfWorkaround.h"
 
 #include <KoXmlReader.h>
 #include <KoXmlWriter.h>
@@ -70,27 +71,31 @@ class KoShape::Private
 public:
     Private(KoShape *shape)
             : size(50, 50),
-            zIndex(0),
             parent(0),
+            userData(0),
+            appData(0),
+            fill(0),
+            border(0),
+            me(shape),
+            shadow(0),
+            zIndex(0),
             visible(true),
             printable(true),
             locked(false),
             keepAspect(false),
             selectable(true),
             detectCollision(false),
-            userData(0),
-            appData(0),
-            fill(0),
-            border(0),
-            me(shape),
-            shadow(0) {
+            protectContent(false)
+    {
     }
 
     ~Private() {
         if (parent)
             parent->removeChild(me);
-        foreach(KoShapeManager *manager, shapeManagers)
+        foreach(KoShapeManager *manager, shapeManagers) {
             manager->remove(me);
+            manager->removeAdditional(me);
+        }
         delete userData;
         delete appData;
         if (border && ! border->removeUser())
@@ -118,16 +123,7 @@ public:
 
     QVector<QPointF> connectors; // in pt
 
-    int zIndex;
     KoShapeContainer *parent;
-
-    bool visible;
-    bool printable;
-    bool locked;
-    bool keepAspect;
-    bool selectable;
-    bool detectCollision;
-
     QSet<KoShapeManager *> shapeManagers;
     KoShapeUserData *userData;
     KoShapeApplicationData *appData;
@@ -137,7 +133,18 @@ public:
     QList<KoShape*> dependees; ///< list of shape dependent on this shape
     KoShapeShadow * shadow; ///< the current shape shadow
     QMap<QByteArray, QString> additionalAttributes;
+    QMap<QByteArray, QString> additionalStyleAttributes;
     QList<KoEventAction *> eventActions; ///< list of event actions the shape has
+
+    uint zIndex : 10; // should be enough ;)
+    uint visible : 1;
+    uint printable : 1;
+    uint locked : 1;
+    uint keepAspect : 1;
+    uint selectable : 1;
+    uint detectCollision : 1;
+    uint protectContent : 1;
+    uint reserved : 15; // for future extensions, keep total bits used 32!
 };
 
 KoShape::KoShape()
@@ -379,7 +386,7 @@ void KoShape::setParent(KoShapeContainer *parent)
 int KoShape::zIndex() const
 {
     if (parent()) // we can't be under our parent...
-        return qMax(d->zIndex, parent()->zIndex());
+        return qMax((int) d->zIndex, parent()->zIndex());
     return d->zIndex;
 }
 
@@ -643,6 +650,16 @@ bool KoShape::isLocked() const
     return d->locked;
 }
 
+void KoShape::setContentProtected(bool protect)
+{
+    d->protectContent = protect;
+}
+
+bool KoShape::isContentProtected() const
+{
+    return d->protectContent;
+}
+
 KoShapeContainer *KoShape::parent() const
 {
     return d->parent;
@@ -754,14 +771,8 @@ bool KoShape::isEditable() const
     if (!d->visible || d->locked)
         return false;
 
-    KoShapeContainer * p = parent();
-    if (p && p->isChildLocked(this))
+    if (d->parent && d->parent->isChildLocked(this))
         return false;
-    while (p) {
-        if (! p->isEditable())
-            return false;
-        p = p->parent();
-    }
 
     return true;
 }
@@ -774,18 +785,41 @@ QString KoShape::saveStyle(KoGenStyle &style, KoShapeSavingContext &context) con
     if (b) {
         b->fillStyle(style, context);
     }
+    else {
+        style.addProperty( "draw:stroke", "none" );
+    }
     KoShapeShadow * s = shadow();
     if (s)
         s->fillStyle(style, context);
 
     KoShapeBackground * bg = background();
-    if (bg)
+    if (bg) {
         bg->fillStyle(style, context);
+    }
+    else {
+        style.addProperty( "draw:fill", "none" );
+    }
 
     if (context.isSet(KoShapeSavingContext::AutoStyleInStyleXml)) {
         style.setAutoStyleInStylesDotXml(true);
     }
 
+    QString value;
+    if (isLocked())
+        value = "position size";
+    if (isContentProtected()) {
+        if (! value.isEmpty())
+            value += ' ';
+        value += "content";
+    }
+    if (!value.isEmpty())
+        d->additionalStyleAttributes.insert("style:protect", value);
+
+    QMap<QByteArray, QString>::const_iterator it(d->additionalStyleAttributes.constBegin());
+    for (; it != d->additionalStyleAttributes.constEnd(); ++it) {
+        style.addProperty(it.key(), it.value());
+    }
+
     return context.mainStyles().lookup(style, context.isSet(KoShapeSavingContext::PresentationShape) ? "pr" : "gr");
 }
 
@@ -850,13 +884,14 @@ bool KoShape::loadOdfAttributes(const KoXmlElement & element, KoShapeLoadingCont
 
     if (attributes & OdfZIndex) {
         if (element.hasAttributeNS(KoXmlNS::draw, "z-index")) {
-            context.addShapeZIndex(this, element.attributeNS(KoXmlNS::draw, "z-index").toInt());
+            setZIndex(element.attributeNS(KoXmlNS::draw, "z-index").toInt());
+        }
+        else {
+            setZIndex(context.zIndex());
         }
-        setZIndex(context.zIndex());
     }
 
     if (attributes & OdfName) {
-
         if (element.hasAttributeNS(KoXmlNS::draw, "name")) {
             setName(element.attributeNS(KoXmlNS::draw, "name"));
         }
@@ -932,19 +967,33 @@ KoShapeBackground * KoShape::loadOdfFill(const KoXmlElement & element, KoShapeLo
 KoShapeBorderModel * KoShape::loadOdfStroke(const KoXmlElement & element, KoShapeLoadingContext & context)
 {
     KoStyleStack &styleStack = context.odfLoadingContext().styleStack();
+    KoOdfStylesReader &stylesReader = context.odfLoadingContext().stylesReader();
+    
     QString stroke = getStyleProperty("stroke", element, context);
     if (stroke == "solid" || stroke == "dash") {
-        QPen pen = KoOdfGraphicStyles::loadOasisStrokeStyle(styleStack, stroke, context.odfLoadingContext().stylesReader());
+        QPen pen = KoOdfGraphicStyles::loadOdfStrokeStyle(styleStack, stroke, stylesReader);
 
         KoLineBorder * border = new KoLineBorder();
+        
+        if (styleStack.hasProperty(KoXmlNS::koffice, "stroke-gradient")) {
+            QString gradientName = styleStack.property(KoXmlNS::koffice, "stroke-gradient");
+            QBrush brush = KoOdfGraphicStyles::loadOdfGradientStyleByName(stylesReader, gradientName, size());
+            border->setLineBrush(brush);
+        } else {
+            border->setColor(pen.color());
+        }
+
+#ifndef NWORKAROUND_ODF_BUGS
+        KoOdfWorkaround::fixPenWidth(pen, context);
+#endif
         border->setLineWidth(pen.widthF());
-        border->setColor(pen.color());
         border->setJoinStyle(pen.joinStyle());
         border->setLineStyle(pen.style(), pen.dashPattern());
 
         return border;
-    } else
+    } else {
         return 0;
+    }
 }
 
 KoShapeShadow * KoShape::loadOdfShadow(const KoXmlElement & element, KoShapeLoadingContext & context)
@@ -975,8 +1024,8 @@ QMatrix KoShape::parseOdfTransform(const QString &transform)
 
     // Split string for handling 1 transform statement at a time
     QStringList subtransforms = transform.split(')', QString::SkipEmptyParts);
-    QStringList::ConstIterator it = subtransforms.begin();
-    QStringList::ConstIterator end = subtransforms.end();
+    QStringList::ConstIterator it = subtransforms.constBegin();
+    QStringList::ConstIterator end = subtransforms.constEnd();
     for (; it != end; ++it) {
         QStringList subtransform = (*it).split('(', QString::SkipEmptyParts);
 
@@ -1038,7 +1087,8 @@ QMatrix KoShape::parseOdfTransform(const QString &transform)
         } else if (cmd == "matrix") {
             QMatrix m;
             if (params.count() >= 6)
-                m.setMatrix(params[0].toDouble(), params[1].toDouble(), params[2].toDouble(), params[3].toDouble(), KoUnit::parseValue(params[4]), KoUnit::parseValue(params[5]));
+                m.setMatrix(params[0].toDouble(), params[1].toDouble(), params[2].toDouble(), params[3].toDouble(),
+                        KoUnit::parseValue(params[4]), KoUnit::parseValue(params[5]));
             matrix = matrix * m;
         }
     }
@@ -1085,19 +1135,27 @@ void KoShape::saveOdfAttributes(KoShapeSavingContext &context, int attributes) c
     // The position is implicitly stored in the transformation matrix
     // if the transformation is saved as well
     if ((attributes & OdfPosition) && !(attributes & OdfTransformation)) {
-        const QPointF p(position());
+        const QPointF p(position() * context.shapeOffset(this));
         context.xmlWriter().addAttributePt("svg:x", p.x());
         context.xmlWriter().addAttributePt("svg:y", p.y());
     }
 
     if (attributes & OdfTransformation) {
-        QMatrix matrix = absoluteTransformation(0);
+        QMatrix matrix = absoluteTransformation(0) * context.shapeOffset(this);
         if (! matrix.isIdentity()) {
-            QString m = QString("matrix(%1 %2 %3 %4 %5pt %6pt)")
-                        .arg(matrix.m11()).arg(matrix.m12())
-                        .arg(matrix.m21()).arg(matrix.m22())
-                        .arg(matrix.dx()) .arg(matrix.dy());
-            context.xmlWriter().addAttribute("draw:transform", m);
+            if (qAbs(matrix.m11() - 1) < 1E-5           // 1
+                    && qAbs(matrix.m12()) < 1E-5        // 0
+                    && qAbs(matrix.m21()) < 1E-5        // 0
+                    && qAbs(matrix.m22() - 1) < 1E-5) { // 1
+                context.xmlWriter().addAttribute("svg:x", QString("%1pt").arg(matrix.dx()));
+                context.xmlWriter().addAttribute("svg:y", QString("%1pt").arg(matrix.dy()));
+            } else {
+                QString m = QString("matrix(%1 %2 %3 %4 %5pt %6pt)")
+                            .arg(matrix.m11()).arg(matrix.m12())
+                            .arg(matrix.m21()).arg(matrix.m22())
+                            .arg(matrix.dx()) .arg(matrix.dy());
+                context.xmlWriter().addAttribute("draw:transform", m);
+            }
         }
     }
 
@@ -1185,6 +1243,12 @@ void KoShape::notifyShapeChanged(KoShape * shape, ChangeType type)
     Q_UNUSED(type);
 }
 
+void KoShape::notifyChangedShape(ChangeType type)
+{
+    d->shapeChanged(type);
+}
+
+
 KoSnapData KoShape::snapData() const
 {
     return KoSnapData();
@@ -1200,12 +1264,23 @@ void KoShape::removeAdditionalAttribute(const char * name)
     d->additionalAttributes.remove(name);
 }
 
-bool KoShape::hasAdditionalAttribute(const char * name)
+bool KoShape::hasAdditionalAttribute(const char * name) const
 {
     return d->additionalAttributes.contains(name);
 }
 
-QString KoShape::additionalAttribute(const char * name)
+QString KoShape::additionalAttribute(const char * name) const
 {
     return d->additionalAttributes.value(name);
 }
+
+void KoShape::setAdditionalStyleAttribute(const char * name, const QString & value)
+{
+    d->additionalStyleAttributes.insert(name, value);
+}
+
+void KoShape::removeAdditionalStyleAttribute(const char * name)
+{
+    d->additionalStyleAttributes.remove(name);
+}
+
