@@ -1,7 +1,7 @@
 /* This file is part of the KDE project
    Copyright (C) 2006-2008 Thorsten Zachmann <zachmann@kde.org>
    Copyright (C) 2006, 2008 Casper Boemann <cbr@boemann.dk>
-   Copyright (C) 2006 Thomas Zander <zander@kde.org>
+   Copyright (C) 2006-2009 Thomas Zander <zander@kde.org>
    Copyright (C) 2007 Jan Hambrecht <jaham@gmx.net>
 
    This library is free software; you can redistribute it and/or
@@ -117,7 +117,8 @@ public:
         Deleted, ///< the shape was deleted
         BorderChanged, ///< the shapes border has changed
         BackgroundChanged, ///< the shapes background has changed
-        ShadowChanged ///< the shapes shadow has changed
+        ShadowChanged, ///< the shapes shadow has changed
+        ParameterChanged ///< the shapes parameter has changed (KoParameterShape only)
     };
 
     /**
@@ -403,28 +404,41 @@ public:
     /**
      * Makes it possible for the user to select this shape.
      * This parameter defaults to true.
-     * @param selectable when true; set the shape to be selectable.
+     * @param selectable when true; set the shape to be selectable by the user.
      */
     void setSelectable(bool selectable);
     /**
-     * Returns wheather this shape can be selected by the user.
+     * Returns if this shape can be selected by the user.
      * @return true only when the object is selectable.
      */
     bool isSelectable() const;
 
     /**
      * Changes the Shape to be locked in place.
-     * Being locked means the shape can no longer change shape or position.
+     * Being locked means the user can not change shape or position of the shape.
      * @param locked when true; set the shape to be locked.
      */
     void setLocked(bool locked);
     /**
      * Returns current locked state of this shape.
-     * Being locked means the shape can no longer change shape or position.
+     * Being locked means the user can not change shape or position of the shape.
      * @return current locked state of this shape.
      */
     bool isLocked() const;
 
+    /**
+     * Marks the shape to have its content protected against editing.
+     * Content protection is a hint for tools to disallow the user editing the content.
+     * @param protect when true set the shapes content to be protected from user modification.
+     */
+    void setContentProtected(bool protect);
+    /**
+     * Returns current content protection state of this shape.
+     * Content protection is a hint for tools to disallow the user editing the content.
+     * @return current content protection state of this shape.
+     */
+    bool isContentProtected() const;
+
     /**
      * Returns the parent, or 0 if there is no parent.
      * @return the parent, or 0 if there is no parent.
@@ -678,9 +692,10 @@ public:
     void notifyChanged();
 
     /**
-     * A shape can be in a state that its doing loading or text layout or similar in which case
+     * A shape can be in a state that it is doing processing data like loading or text layout.
+     * In which case
      * it can be shown on screen just fine (probably partially) but it should really not be printed
-     * until its fully done.
+     * until it is fully done processing.
      * Flake will call this method from a non-main thread and only start printing it when the
      * method returned.
      * Warning!  This method can be blocking for a long time, never call it on the gui-thread!
@@ -690,10 +705,13 @@ public:
     /**
      * Schedule the shape for thread-safe deletion.
      * After calling this method will self-delete in the main threads event loop.
-     * If deleting a shape can possibly be done in a separate thread, you should delete it
-     * using this method.  If you delete a shape from another thread then its possible the main
+     * If your code deletes a shape and your code can possibly be running in a separate thread,
+     * you should use this method to delete the shape.
+     * The reason for this is that If you delete a shape from another thread then it is
+     * possible the main
      * thread will use it after its been removed, while painting for example.
-     * Note that in contrary to the equivalent method on QObject, you can not call this more than ones!
+     *
+     * Note that in contrary to the equivalent method on QObject, you can not call this more than once.
      */
     void deleteLater();
 
@@ -737,7 +755,7 @@ public:
      *
      * @return true if there is a attribute with prefix:tag set, false otherwise
      */
-    bool hasAdditionalAttribute(const char * name);
+    bool hasAdditionalAttribute(const char * name) const;
 
     /**
      * Get additional attribute
@@ -746,7 +764,11 @@ public:
      *
      * @return The value of the attribute if it exists or a null string if not found.
      */
-    QString additionalAttribute(const char * name);
+    QString additionalAttribute(const char * name) const;
+
+    void setAdditionalStyleAttribute(const char * name, const QString & value);
+
+    void removeAdditionalStyleAttribute(const char * name);
 
 protected:
 
@@ -847,6 +869,7 @@ protected:
      * @param detect if true detect collisions.
      */
     void setCollisionDetection(bool detect);
+
     /**
      * get the property collision detection.
      * @returns true if collision detection is on.
@@ -856,6 +879,12 @@ protected:
     /// return the current matrix that contains the rotation/scale/position of this shape
     const QMatrix& matrix() const;
 
+    /**
+     * Notify the shape that a change was done. To be used by inheriting shapes.
+     * @param type the change type
+     */
+    void notifyChangedShape(ChangeType type);
+
 private:
     friend class KoShapeManager;
     void addShapeManager(KoShapeManager * manager);
