@@ -1,6 +1,6 @@
 /* This file is part of the KDE project
 
-   Copyright (C) 2006 Thorsten Zachmann <zachmann@kde.org>
+   Copyright (C) 2006-2008 Thorsten Zachmann <zachmann@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -26,14 +26,13 @@
 #include <QSet>
 
 #include "KoFlake.h"
-#include <flake_export.h>
+#include "flake_export.h"
 #include <KoRTree.h>
 
 class KoShape;
 class KoSelection;
 class KoViewConverter;
 class KoCanvasBase;
-class KoShapeConnection;
 class KoPointerEvent;
 class KoShapeManagerPaintingStrategy;
 
@@ -71,7 +70,7 @@ public:
      * @param shapes the new shapes to manage.
      * @param repaint if true it will trigger a repaint of the shapes
      */
-    void setShapes( const QList<KoShape *> &shapes, bool repaint = true );
+    void setShapes(const QList<KoShape *> &shapes, bool repaint = true);
 
     /// returns the list of maintained shapes
     const QList<KoShape *> & shapes() const;
@@ -83,12 +82,26 @@ public:
      */
     void add(KoShape *shape);
 
+    /**
+     * Add an additional shape to the manager.
+     *
+     * For additional shapes only updates are handled
+     */
+    void addAdditional(KoShape *shape);
+
     /**
      * Remove a KoShape from this manager
      * @param shape the shape to remove
      */
     void remove(KoShape *shape);
 
+    /**
+     * Remove an additional shape
+     *
+     * For additional shapes only updates are handled
+     */
+    void removeAdditional(KoShape *shape);
+
     /// return the selection shapes for this shapeManager
     KoSelection * selection() const;
 
@@ -98,7 +111,7 @@ public:
      * @param forPrint if true, make sure only actual content is drawn and no decorations.
      * @param converter to convert between document and view coordinates.
      */
-    virtual void paint( QPainter &painter, const KoViewConverter &converter, bool forPrint );
+    virtual void paint(QPainter &painter, const KoViewConverter &converter, bool forPrint);
 
     /**
      * Returns the shape located at a specific point in the document.
@@ -108,14 +121,14 @@ public:
      * @param selection controls which shape is returned when more than one shape is at the specific point
      * @param omitHiddenShapes if true, only visible shapes are considered
      */
-    KoShape * shapeAt( const QPointF &position, KoFlake::ShapeSelection selection = KoFlake::ShapeOnTop, bool omitHiddenShapes = true );
+    KoShape * shapeAt(const QPointF &position, KoFlake::ShapeSelection selection = KoFlake::ShapeOnTop, bool omitHiddenShapes = true);
 
     /**
      * Returns the shapes which intersects the specific rect in the document.
      * @param rect the rectangle in the document coordinate system.
      * @param omitHiddenShapes if true, only visible shapes are considered
      */
-    QList<KoShape *> shapesAt( const QRectF &rect, bool omitHiddenShapes = true );
+    QList<KoShape *> shapesAt(const QRectF &rect, bool omitHiddenShapes = true);
 
     /**
      * Request a repaint to be queued.
@@ -129,7 +142,7 @@ public:
      * @param selectionHandles if true; find out if the shape is selected and repaint its
      *   selection handles at the same time.
      */
-    void update( QRectF &rect, const KoShape *shape = 0, bool selectionHandles = false );
+    void update(QRectF &rect, const KoShape *shape = 0, bool selectionHandles = false);
 
     /**
      * Update the tree for finding the shapes.
@@ -138,13 +151,7 @@ public:
      * will be merged into one.
      * @param shape the shape to updated its position in the tree.
      */
-    void notifyShapeChanged( KoShape * shape );
-
-    /**
-     * Add a shape connection to the manager so it can be taken into account for drawing purposes.
-     * Note that this is typically called by the shape instance only.
-     */
-    void addShapeConnection(KoShapeConnection *connection);
+    void notifyShapeChanged(KoShape * shape);
 
     /**
      * Switch to editing the shape that is at the position of the event.
@@ -155,25 +162,25 @@ public:
     void suggestChangeTool(KoPointerEvent *event);
 
     /**
-     * Paint a shape 
+     * Paint a shape
      *
      * @param shape the shape to paint
      * @param painter the painter to paint to.
      * @param converter to convert between document and view coordinates.
      * @param forPrint if true, make sure only actual content is drawn and no decorations.
      */
-    void paintShape( KoShape * shape, QPainter &painter, const KoViewConverter &converter, bool forPrint );
+    void paintShape(KoShape * shape, QPainter &painter, const KoViewConverter &converter, bool forPrint);
 
     /**
      * Set the strategy of the KoShapeManager
      *
      * This can be used to change the behaviour of the painting of the shapes.
-     * @param strategy the new strategy. The given strategy has to be allocared 
+     * @param strategy the new strategy. The given strategy has to be allocared
      *        on the stack and you the ownership will be taken by the shape manager.
-     *        The shape manager will delete the last set strategy when you set a 
+     *        The shape manager will delete the last set strategy when you set a
      *        new one.
      */
-    void setPaintingStrategy( KoShapeManagerPaintingStrategy * strategy );
+    void setPaintingStrategy(KoShapeManagerPaintingStrategy * strategy);
 
 private:
     /**
@@ -181,7 +188,7 @@ private:
      * @param shape the shape to add
      * @param repaint if true it will tigger a repaint of the shape
      */
-    void add(KoShape *shape, bool repaint );
+    void add(KoShape *shape, bool repaint);
 
     /**
      * Update the tree when there are shapes in m_aggregate4update. This is done so not all
@@ -192,6 +199,8 @@ private:
 signals:
     /// emitted when the selection is changed
     void selectionChanged();
+    /// emitted when an object in the selection is changed (moved/rotated etc)
+    void selectionContentChanged();
 
 private:
     class Private;
