@@ -37,7 +37,7 @@
 class KoCharacterStyle::Private
 {
 public:
-    Private() { }
+    Private();
     ~Private() { }
 
     void setProperty(int key, const QVariant &value) {
@@ -74,10 +74,58 @@ public:
         return variant.value<QColor>();
     }
 
+    //Overload the hard-coded default with defaultstyles.xml properties if defined
+    void setApplicationDefaults(KoOdfLoadingContext &context);
+
+    //This should be called after all charFormat properties are merged to the cursor.
+    void ensureMinimalProperties(QTextCursor &cursor, bool blockCharFormatAlso);
+
+    StylePrivate hardCodedDefaultStyle;
+
     QString name;
     StylePrivate stylesPrivate;
 };
 
+KoCharacterStyle::Private::Private()
+{
+    //set the minimal default properties
+    hardCodedDefaultStyle.add(QTextFormat::FontFamily, QString("Sans Serif"));
+    hardCodedDefaultStyle.add(QTextFormat::FontPointSize, 12.0);
+    hardCodedDefaultStyle.add(QTextFormat::ForegroundBrush, QBrush(Qt::black));
+}
+
+void KoCharacterStyle::Private::setApplicationDefaults(KoOdfLoadingContext &context)
+{
+    KoStyleStack defaultStyleStack;
+    const KoXmlElement* appDef = context.defaultStylesReader().defaultStyle("paragraph");
+    if (appDef) {
+        defaultStyleStack.push(*appDef);
+        defaultStyleStack.setTypeProperties("text");
+        KoCharacterStyle defStyle;
+        defStyle.loadOdfProperties(defaultStyleStack);
+
+        QList<int> keys = defStyle.d->stylesPrivate.keys();
+        foreach(int key, keys) {
+            hardCodedDefaultStyle.add(key, defStyle.value(key));
+        }
+    }
+}
+
+void KoCharacterStyle::Private::ensureMinimalProperties(QTextCursor &cursor, bool blockCharFormatAlso)
+{
+    QTextCharFormat format = cursor.charFormat();
+    QList<int> keys = hardCodedDefaultStyle.keys();
+    for (int i = 0; i < keys.count(); i++) {
+        QVariant variant = hardCodedDefaultStyle.value(keys.at(i));
+        if (!variant.isNull() && !format.hasProperty(keys.at(i))) {
+            format.setProperty(keys.at(i), variant);
+        }
+    }
+    cursor.mergeCharFormat(format);
+    if (blockCharFormatAlso)
+        cursor.mergeBlockCharFormat(format);
+}
+
 KoCharacterStyle::KoCharacterStyle(QObject *parent)
         : QObject(parent), d(new Private())
 {
@@ -172,6 +220,7 @@ void KoCharacterStyle::applyStyle(QTextBlock &block) const
     applyStyle(cf);
     cursor.mergeCharFormat(cf);
     cursor.mergeBlockCharFormat(cf);
+    d->ensureMinimalProperties(cursor, true);
 }
 
 void KoCharacterStyle::applyStyle(QTextCursor *selection) const
@@ -179,6 +228,7 @@ void KoCharacterStyle::applyStyle(QTextCursor *selection) const
     QTextCharFormat cf;
     applyStyle(cf);
     selection->mergeCharFormat(cf);
+    d->ensureMinimalProperties(*selection, false);
 }
 
 // OASIS 14.2.29
@@ -319,7 +369,6 @@ static QString exportOdfLineWidth(KoCharacterStyle::LineWeight lineWeight, qreal
     }
 }
 
-#if QT_VERSION >= KDE_MAKE_VERSION(4,5,0)
 static QString exportOdfFontStyleHint(QFont::StyleHint hint)
 {
     switch (hint) {
@@ -338,7 +387,6 @@ static QString exportOdfFontStyleHint(QFont::StyleHint hint)
         return "";
     }
 }
-#endif
 
 void KoCharacterStyle::setFontFamily(const QString &family)
 {
@@ -388,7 +436,6 @@ bool KoCharacterStyle::fontFixedPitch() const
 {
     return d->propertyBoolean(QTextFormat::FontFixedPitch);
 }
-#if QT_VERSION >= KDE_MAKE_VERSION(4,5,0)
 void KoCharacterStyle::setFontStyleHint(QFont::StyleHint styleHint)
 {
     d->setProperty(QTextFormat::FontStyleHint, styleHint);
@@ -405,7 +452,6 @@ bool KoCharacterStyle::fontKerning() const
 {
     return d->propertyBoolean(QTextFormat::FontKerning);
 }
-#endif
 void KoCharacterStyle::setVerticalAlignment(QTextCharFormat::VerticalAlignment alignment)
 {
     d->setProperty(QTextFormat::TextVerticalAlignment, alignment);
@@ -642,8 +688,34 @@ bool KoCharacterStyle::hasProperty(int key) const
 //in 1.6 this was defined in KoTextFormat::load(KoOasisContext& context)
 void KoCharacterStyle::loadOdf(KoOdfLoadingContext& context)
 {
+    d->setApplicationDefaults(context);
     KoStyleStack &styleStack = context.styleStack();
+    loadOdfProperties(styleStack);
 
+    QString fontName;
+    if (styleStack.hasProperty(KoXmlNS::style, "font-name")) {
+        // This font name is a reference to a font face declaration.
+        KoOdfStylesReader &stylesReader = context.stylesReader();
+        const KoXmlElement *fontFace = stylesReader.findStyle(styleStack.property(KoXmlNS::style, "font-name"));
+        if (fontFace != 0)
+            fontName = fontFace->attributeNS(KoXmlNS::svg, "font-family", "");
+    }
+    if (! fontName.isNull()) {
+    // Hmm, the remove "'" could break it's in the middle of the fontname...
+        fontName = fontName.remove('\'');
+
+    // 'Thorndale' is not known outside OpenOffice so we substitute it
+    // with 'Times New Roman' that looks nearly the same.
+        if (fontName == "Thorndale")
+            fontName = "Times New Roman";
+
+        fontName.remove(QRegExp("\\sCE$")); // Arial CE -> Arial
+        setFontFamily(fontName);
+    }
+}
+
+void KoCharacterStyle::loadOdfProperties(KoStyleStack &styleStack)
+{
     // The fo:color attribute specifies the foreground color of text.
     if (styleStack.hasProperty(KoXmlNS::fo, "color")) {     // 3.10.3
         if (styleStack.property(KoXmlNS::style, "use-window-font-color") != "true") {
@@ -667,7 +739,6 @@ void KoCharacterStyle::loadOdf(KoOdfLoadingContext& context)
                 setFontFixedPitch(false);
         }
 
-#if QT_VERSION >= KDE_MAKE_VERSION(4,5,0)
         if (styleStack.hasProperty(KoXmlNS::style, "font-family-generic")) {
             QString genericFamily = styleStack.property(KoXmlNS::style, "font-family-generic");
             if (genericFamily == "roman")
@@ -680,10 +751,10 @@ void KoCharacterStyle::loadOdf(KoOdfLoadingContext& context)
                 setFontStyleHint(QFont::Decorative);
             else if (genericFamily == "system")
                 setFontStyleHint(QFont::System);
-            else if (genericFamily == "script")
-                ; // TODO: no hint available in Qt yet
+            else if (genericFamily == "script") {
+                ; // TODO: no hint available in Qt yet, we should at least store it as a property internally!
+            }
         }
-#endif
 
         if (styleStack.hasProperty(KoXmlNS::style, "font-charset")) {
             // this property is not required by Qt, since Qt auto selects the right font based on the text
@@ -693,13 +764,6 @@ void KoCharacterStyle::loadOdf(KoOdfLoadingContext& context)
     }
     if (styleStack.hasProperty(KoXmlNS::style, "font-family"))
         fontName = styleStack.property(KoXmlNS::style, "font-family");
-    if (styleStack.hasProperty(KoXmlNS::style, "font-name")) {
-        // This font name is a reference to a font face declaration.
-        KoOdfStylesReader &stylesReader = context.stylesReader();
-        const KoXmlElement *fontFace = stylesReader.findStyle(styleStack.property(KoXmlNS::style, "font-name"));
-        if (fontFace != 0)
-            fontName = fontFace->attributeNS(KoXmlNS::svg, "font-family", "");
-    }
 
     if (! fontName.isNull()) {
         // Hmm, the remove "'" could break it's in the middle of the fontname...
@@ -920,7 +984,6 @@ void KoCharacterStyle::loadOdf(KoOdfLoadingContext& context)
         }
     }
 
-#if QT_VERSION >= KDE_MAKE_VERSION(4,5,0)
     if (styleStack.hasProperty(KoXmlNS::style, "letter-kerning")) {
         if (styleStack.property(KoXmlNS::style, "letter-kerning") == "true") {
             setFontKerning(true);
@@ -928,11 +991,10 @@ void KoCharacterStyle::loadOdf(KoOdfLoadingContext& context)
             setFontKerning(false);
         }
     }
-#endif
 
     if (styleStack.hasProperty(KoXmlNS::style, "text-outline")) {
         if (styleStack.property(KoXmlNS::style, "text-outline") == "true") {
-            setTextOutline(QPen(foreground(), 0));
+            setTextOutline(QPen((foreground().style() != Qt::NoBrush)?foreground():QBrush(Qt::black) , 0));
             setForeground(Qt::transparent);
         } else {
             setTextOutline(QPen(Qt::NoPen));
@@ -969,7 +1031,6 @@ void KoCharacterStyle::loadOdf(KoOdfLoadingContext& context)
     generateKey();
     addRef();
 #endif
-
 }
 
 bool KoCharacterStyle::operator==(const KoCharacterStyle &other) const
@@ -995,6 +1056,9 @@ bool KoCharacterStyle::isEmpty() const
 
 void KoCharacterStyle::saveOdf(KoGenStyle &style)
 {
+    if (!d->name.isEmpty()) {
+        style.addAttribute("style:display-name", d->name);
+    }
     QList<int> keys = d->stylesPrivate.keys();
     foreach(int key, keys) {
         if (key == QTextFormat::FontWeight) {
@@ -1022,7 +1086,6 @@ void KoCharacterStyle::saveOdf(KoGenStyle &style)
         } else if (key == QTextFormat::FontFixedPitch) {
             bool fixedPitch = d->stylesPrivate.value(key).toBool();
             style.addProperty("style:font-pitch", fixedPitch ? "fixed" : "variable", KoGenStyle::TextType);
-#if QT_VERSION >= KDE_MAKE_VERSION(4,5,0)
         } else if (key == QTextFormat::FontStyleHint) {
             bool ok = false;
             int styleHint = d->stylesPrivate.value(key).toInt(&ok);
@@ -1030,7 +1093,6 @@ void KoCharacterStyle::saveOdf(KoGenStyle &style)
                 style.addProperty("style:font-family-generic", exportOdfFontStyleHint((QFont::StyleHint) styleHint), KoGenStyle::TextType);
         } else if (key == QTextFormat::FontKerning) {
             style.addProperty("style:letter-kerning", fontKerning() ? "true" : "false", KoGenStyle::TextType);
-#endif
         } else if (key == QTextFormat::FontCapitalization) {
             switch (fontCapitalization()) {
             case QFont::SmallCaps:
