@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
- * Copyright (C) 2006-2007 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2006-2007, 2009 Thomas Zander <zander@kde.org>
  * Copyright (C) 2008 Girish Ramakrishnan <girish@forwardbias.in>
  *
  * This library is free software; you can redistribute it and/or
@@ -198,6 +198,9 @@ void KoListStyle::loadOdf(KoOdfLoadingContext& context, const KoXmlElement& styl
 
 void KoListStyle::saveOdf(KoGenStyle &style)
 {
+    if (!d->name.isEmpty()) {
+        style.addAttribute("style:display-name", d->name);
+    }
     QBuffer buffer;
     buffer.open(QIODevice::WriteOnly);
     KoXmlWriter elementWriter(&buffer);    // TODO pass indentation level
@@ -218,3 +221,8 @@ bool KoListStyle::isNumberingStyle(int style)
              || style == KoListStyle::BallotXItem || style == KoListStyle::RightArrowItem 
              || style == KoListStyle::RightArrowHeadItem);
 }
+
+QList<int> KoListStyle::listLevels() const
+{
+    return d->levels.keys();
+}
