@@ -35,7 +35,8 @@ class KoListStyle;
 class QTextBlock;
 class KoStyleStack;
 class KoGenStyle;
-class KoXmlElement;
+class KoGenStyles;
+#include "KoXmlReaderForward.h"
 class KoOdfLoadingContext;
 
 /**
@@ -537,7 +538,7 @@ public:
      */
     void loadOdf(const KoXmlElement* element, KoOdfLoadingContext & context);
 
-    void saveOdf(KoGenStyle &style);
+    void saveOdf(KoGenStyle &style, KoGenStyles &mainStyles);
 
     /**
      * Returns true if this paragraph style has the property set.
@@ -578,6 +579,8 @@ private:
 
     class Private;
     Private * const d;
+
+    bool normalLineHeight;
 };
 
 #endif
