@@ -166,11 +166,14 @@ public:
     bool m_storeInternal; // Store this doc internally even if url is external
     bool m_bLoading; // True while loading (openUrl is async)
 
-    KoOpenPane* m_startUpWidget;
+    QPointer<KoOpenPane> m_startUpWidget;
     QString m_templateType;
     QList<KoVersionInfo> m_versionInfo;
 
     KUndoStack* m_undoStack;
+
+    KoGridData gridData;
+    KoGuidesData guidesData;
 };
 
 // Used in singleViewMode
@@ -301,7 +304,7 @@ KoDocument::~KoDocument()
     // Tell our views that the document is already destroyed and
     // that they shouldn't try to access it.
     foreach(KoView* view, d->m_views)
-    view->setDocumentDeleted();
+        view->setDocumentDeleted();
 
     delete d->m_startUpWidget;
     d->m_startUpWidget = 0;
@@ -406,7 +409,7 @@ bool KoDocument::saveFile()
 
     if (backupFile()) {
         if (url().isLocalFile())
-            KSaveFile::backupFile(url().path(), d->m_backupPath);
+            KSaveFile::backupFile(url().toLocalFile(), d->m_backupPath);
         else {
             KIO::UDSEntry entry;
             if (KIO::NetAccess::stat(url(), entry, shells().current())) {     // this file exists => backup
@@ -627,7 +630,7 @@ void KoDocument::setReadWrite(bool readwrite)
     KParts::ReadWritePart::setReadWrite(readwrite);
 
     foreach(KoView* view, d->m_views)
-    view->updateReadWrite(readwrite);
+        view->updateReadWrite(readwrite);
 
     Q3PtrListIterator<KoDocumentChild> dIt(d->m_children);
     for (; dIt.current(); ++dIt)
@@ -719,18 +722,19 @@ const Q3PtrList<KoDocumentChild>& KoDocument::children() const
 
 KParts::Part *KoDocument::hitTest(QWidget *widget, const QPoint &globalPos)
 {
-    foreach(KoView* view, d->m_views)
-    if (static_cast<QWidget *>(view) == widget) {
-        QPoint canvasPos(view->canvas()->mapFromGlobal(globalPos));
-        canvasPos.rx() += view->canvasXOffset();
-        canvasPos.ry() += view->canvasYOffset();
+    foreach(KoView* view, d->m_views) {
+        if (static_cast<QWidget *>(view) == widget) {
+            QPoint canvasPos(view->canvas()->mapFromGlobal(globalPos));
+            canvasPos.rx() += view->canvasXOffset();
+            canvasPos.ry() += view->canvasYOffset();
 
-        KParts::Part *part = view->hitTest(canvasPos);
-        if (part)
-            return part;
+            KParts::Part *part = view->hitTest(canvasPos);
+            if (part)
+                return part;
+        }
     }
 
-    return 0L;
+    return 0;
 }
 
 KoDocument* KoDocument::hitTest(const QPoint &pos, KoView* view, const QMatrix &matrix)
@@ -1360,7 +1364,7 @@ bool KoDocument::openUrl(const KUrl & _url)
     bool autosaveOpened = false;
     d->m_bLoading = true;
     if (url.isLocalFile() && d->m_shouldCheckAutoSaveFile) {
-        QString file = url.path();
+        QString file = url.toLocalFile();
         QString asf = autoSaveFile(file);
         if (QFile::exists(asf)) {
             //kDebug(30003) <<"KoDocument::openUrl asf=" << asf;
@@ -1637,16 +1641,21 @@ bool KoDocument::oldLoadAndParse(KoStore* store, const QString& filename, KoXmlD
     return true;
 }
 
-bool KoDocument::loadNativeFormat(const QString & file)
+bool KoDocument::loadNativeFormat(const QString & file_)
 {
+    QString file = file_;
     QFileInfo fileInfo(file);
     if (!fileInfo.exists()) { // check duplicated from openUrl, but this is useful for templates
         d->lastErrorMessage = i18n("The file %1 does not exist.", file);
         return false;
     }
     if (!fileInfo.isFile()) {
-        d->lastErrorMessage = i18n("%1 is not a file." , file);
-        return false;
+        file = file += "/content.xml";
+        QFileInfo fileInfo2(file);
+        if (!fileInfo2.exists() || !fileInfo2.isFile()) {
+            d->lastErrorMessage = i18n("%1 is not a file." , file_);
+            return false;
+        }
     }
 
     QApplication::setOverrideCursor(Qt::WaitCursor);
@@ -2564,12 +2573,12 @@ void KoDocument::openExistingFile(const KUrl& url)
     setModified(false);
 
     if (ok)
-        QTimer::singleShot(0, this, SLOT(deleteOpenPane()));
+        deleteOpenPane();
 }
 
 void KoDocument::openTemplate(const KUrl& url)
 {
-    bool ok = loadNativeFormat(url.path());
+    bool ok = loadNativeFormat(url.toLocalFile());
     setModified(false);
 
     if (ok) {
@@ -2627,7 +2636,7 @@ void KoDocument::deleteOpenPane()
 {
     if (d->m_startUpWidget) {
         d->m_startUpWidget->hide();
-        QTimer::singleShot(1000, this, SLOT(deleteOpenPaneDelayed()));
+        d->m_startUpWidget->deleteLater();
 
         shells().getFirst()->factory()->container("mainToolBar", shells().getFirst())->show();
         shells().getFirst()->setRootDocument(this);
@@ -2636,12 +2645,6 @@ void KoDocument::deleteOpenPane()
     }
 }
 
-void KoDocument::deleteOpenPaneDelayed()
-{
-    delete d->m_startUpWidget;
-    d->m_startUpWidget = 0;
-}
-
 QList<KoDocument::CustomDocumentWidgetItem> KoDocument::createCustomDocumentWidgets(QWidget * /*parent*/)
 {
     return QList<CustomDocumentWidgetItem>();
@@ -2697,5 +2700,15 @@ void KoDocument::clearUndoHistory()
     d->m_undoStack->clear();
 }
 
+KoGridData &KoDocument::gridData()
+{
+    return d->gridData;
+}
+
+KoGuidesData &KoDocument::guidesData()
+{
+    return d->guidesData;
+}
+
 #include "KoDocument_p.moc"
 #include "KoDocument.moc"
