@@ -17,7 +17,6 @@
  */
 #ifndef KIS_STRATEGY_COLORSPACE_GRAYSCALE_U16_H_
 #define KIS_STRATEGY_COLORSPACE_GRAYSCALE_U16_H_
-#include <QColor>
 
 #include <klocale.h>
 #include "pigment_gray_u16_export.h"
@@ -30,10 +29,13 @@ typedef KoColorSpaceTrait<quint16, 2, 1> GrayAU16Traits;
 class PIGMENT_GRAY_U16_EXPORT KisGrayAU16ColorSpace : public KoLcmsColorSpace<GrayAU16Traits>
 {
     public:
-        KisGrayAU16ColorSpace(KoColorSpaceRegistry * parent, KoColorProfile *p);
+        KisGrayAU16ColorSpace( KoColorProfile *p);
         virtual bool willDegrade(ColorSpaceIndependence ) const { return false; }
         virtual KoID colorModelId() const { return GrayAColorModelID; }
         virtual KoID colorDepthId() const { return Integer16BitsColorDepthID; }
+        virtual KoColorSpace* clone() const;
+        virtual void colorToXML( const quint8* pixel, QDomDocument& doc, QDomElement& colorElt) const;
+        virtual void colorFromXML( quint8* pixel, const QDomElement& elt) const;
 };
 
 class KisGrayAU16ColorSpaceFactory : public KoLcmsColorSpaceFactory
@@ -48,7 +50,7 @@ public:
     virtual int referenceDepth() const { return 16; }
     virtual bool userVisible() const { return true; }
 
-    virtual KoColorSpace *createColorSpace(KoColorSpaceRegistry * parent, KoColorProfile *p) const { return new KisGrayAU16ColorSpace(parent, p); }
+    virtual KoColorSpace *createColorSpace( const KoColorProfile *p) const { return new KisGrayAU16ColorSpace( p->clone()); }
 
     virtual QString defaultProfile() const { return "gray built-in - (lcms internal)"; }
 };
