@@ -176,12 +176,7 @@ bool Layout::addLine(QTextLine &line)
         if (oldFootnoteDocLength >= 0) {
             QTextCursor cursor(m_textShape->footnoteDocument());
             cursor.setPosition(oldFootnoteDocLength);
-#if QT_VERSION >= KDE_MAKE_VERSION(4,5,0)
             cursor.setPosition(m_textShape->footnoteDocument()->characterCount()-1, QTextCursor::KeepAnchor);
-#else
-            QTextBlock last = m_textShape->footnoteDocument()->end().previous();
-            cursor.setPosition(last.position() + last.length() - 1, QTextCursor::KeepAnchor);
-#endif
             cursor.removeSelectedText();
         }
 
@@ -1136,7 +1131,7 @@ void Layout::drawListItem(QPainter *painter, const QTextBlock &block)
             Qt::Alignment align = static_cast<Qt::Alignment>(listFormat.intProperty(KoListStyle::Alignment));
             if (align == 0)
                 align = Qt::AlignLeft;
-            else if (align != Qt::AlignAuto)
+            else if (align != Qt::AlignLeft)
                 align |= Qt::AlignAbsolute;
             QTextOption option(align);
             option.setTextDirection(block.layout()->textOption().textDirection());
@@ -1227,7 +1222,9 @@ void Layout::drawListItem(QPainter *painter, const QTextBlock &block)
             }
             break;
             case KoListStyle::CustomCharItem:
-                painter->drawText(0, 0, QChar(listFormat.intProperty(KoListStyle::BulletCharacter)));
+                if (!QChar(listFormat.intProperty(KoListStyle::BulletCharacter)).isNull()){
+                    painter->drawText(0, 0, QChar(listFormat.intProperty(KoListStyle::BulletCharacter)));
+                }
                 break;
             default:; // others we ignore.
             }
@@ -1331,7 +1328,7 @@ qreal Layout::inlineCharHeight(const QTextFragment &fragment)
 
 qreal Layout::findFootnote(const QTextLine &line, int *oldLength)
 {
-    if (m_parent->inlineObjectTextManager() == 0)
+    if (m_parent->inlineTextObjectManager() == 0)
         return 0;
     Q_ASSERT(oldLength);
     QString text = m_block.text();
@@ -1341,15 +1338,10 @@ qreal Layout::findFootnote(const QTextLine &line, int *oldLength)
         QTextCursor c1(m_block);
         c1.setPosition(m_block.position() + pos);
         c1.setPosition(c1.position() + 1, QTextCursor::KeepAnchor);
-        KoInlineNote *note = dynamic_cast<KoInlineNote*>(m_parent->inlineObjectTextManager()->inlineTextObject(c1));
+        KoInlineNote *note = dynamic_cast<KoInlineNote*>(m_parent->inlineTextObjectManager()->inlineTextObject(c1));
         if (note && note->type() == KoInlineNote::Footnote) {
             QTextCursor cursor(m_textShape->footnoteDocument());
-#if QT_VERSION >= KDE_MAKE_VERSION(4,5,0)
             cursor.setPosition(m_textShape->footnoteDocument()->characterCount()-1);
-#else
-            QTextBlock last = m_textShape->footnoteDocument()->end().previous();
-            cursor.setPosition(last.position() + last.length() - 1);
-#endif
             if (firstFootnote) {
                 (*oldLength) = cursor.position();
                 firstFootnote = false;
