@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
- * Copyright (C) 2006-2007 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2006-2009 Thomas Zander <zander@kde.org>
  * Copyright (C) 2008 Thorsten Zachmann <zachmann@kde.org>
  * Copyright (C) 2008 Girish Ramakrishnan <girish@forwardbias.in>
  *
@@ -40,8 +40,8 @@
 #include <KoPointerEvent.h>
 #include <KoCanvasResourceProvider.h>
 #include <KoColor.h>
-#include <KoColorSetAction.h>
 #include <KoColorBackground.h>
+#include <KoColorPopupAction.h>
 
 #include <KoCharacterStyle.h>
 #include <KoTextDocumentLayout.h>
@@ -324,13 +324,13 @@ TextTool::TextTool(KoCanvasBase *canvas)
     addAction("format_fontsize", m_actionFormatFontSize);
     connect(m_actionFormatFontSize, SIGNAL(fontSizeChanged(int)), &m_selectionHandler, SLOT(setFontSize(int)));
 
-    m_actionFormatTextColor = new KoColorSetAction(this);
+    m_actionFormatTextColor = new KoColorPopupAction(this);
     m_actionFormatTextColor->setIcon(KIcon("format-text-color"));
     m_actionFormatTextColor->setToolTip(i18n("Text Color..."));
     addAction("format_textcolor", m_actionFormatTextColor);
     connect(m_actionFormatTextColor, SIGNAL(colorChanged(const KoColor &)), this, SLOT(setTextColor(const KoColor &)));
 
-    m_actionFormatBackgroundColor = new KoColorSetAction(this);
+    m_actionFormatBackgroundColor = new KoColorPopupAction(this);
     m_actionFormatBackgroundColor->setIcon(KIcon("format-fill-color"));
     m_actionFormatBackgroundColor->setToolTip(i18n("Background Color..."));
     m_actionFormatBackgroundColor->setText(i18n("Background"));
@@ -408,6 +408,17 @@ TextTool::TextTool(KoCanvasBase *canvas)
     connect(m_updateParagDirection.action, SIGNAL(updateUi(const QVariant &)),
             this, SLOT(updateParagraphDirectionUi()));
 
+#ifndef NDEBUG
+    action = new KAction("Paragraph Debug", this); // do NOT add i18n!
+    action->setShortcut(Qt::CTRL + Qt::SHIFT + Qt::Key_P);
+    addAction("detailed_debug_paragraphs", action);
+    connect(action, SIGNAL(triggered()), this, SLOT(debugTextDocument()));
+    action = new KAction("Styles Debug", this); // do NOT add i18n!
+    action->setShortcut(Qt::CTRL + Qt::SHIFT + Qt::ALT + Qt::Key_S);
+    addAction("detailed_debug_styles", action);
+    connect(action, SIGNAL(triggered()), this, SLOT(debugTextStyles()));
+#endif
+
     // setup the context list.
     QSignalMapper *signalMapper = new QSignalMapper(this);
     connect(signalMapper, SIGNAL(mapped(QString)), this, SLOT(startTextEditingPlugin(QString)));
@@ -479,7 +490,6 @@ void TextTool::blinkCaret()
 
 void TextTool::paint(QPainter &painter, const KoViewConverter &converter)
 {
-
     if (m_canvas->canvasWidget()->hasFocus() && !m_caretTimer.isActive()) // make sure we blink
         m_caretTimer.start();
     QTextBlock block = m_caret.block();
@@ -501,7 +511,7 @@ void TextTool::paint(QPainter &painter, const KoViewConverter &converter)
                 continue;
             KoTextShapeData *data = ts->textShapeData();
             // check if shape contains some of the selection, if not, skip
-            if (!( (data->endPosition() >= selectStart && data->position()) <= selectEnd
+            if (!( (data->endPosition() >= selectStart && data->position() <= selectEnd)
                     || (data->position() <= selectStart && data->endPosition() >= selectEnd)) )
                 continue;
             if (painter.hasClipping()) {
@@ -535,7 +545,8 @@ void TextTool::paint(QPainter &painter, const KoViewConverter &converter)
         shapeMatrix.scale(zoomX, zoomY);
         painter.setMatrix(shapeMatrix * painter.matrix());
         painter.translate(0, -data->documentOffset());
-        if (qMin(data->endPosition(), selectEnd) != qMax(data->position(), selectStart)) {
+        if ((data->endPosition() >= selectStart && data->position() <= selectEnd)
+                || (data->position() <= selectStart && data->endPosition() >= selectEnd)) {
             QRectF clip = textRect(qMax(data->position(), selectStart), qMin(data->endPosition(), selectEnd));
             painter.setClipRect(clip, Qt::IntersectClip);
             data->document()->documentLayout()->draw(&painter, pc);
@@ -646,9 +657,12 @@ void TextTool::setShapeData(KoTextShapeData *data)
     bool docChanged = data == 0 || m_textShapeData == 0 || m_textShapeData->document() != data->document();
     if (m_textShapeData && docChanged)
         disconnect(m_textShapeData->document(), SIGNAL(undoCommandAdded()), this, SLOT(addUndoCommand()));
+    if (m_textShapeData)
+        disconnect(m_textShapeData, SIGNAL(destroyed (QObject*)), this, SLOT(shapeDataRemoved()));
     m_textShapeData = data;
     if (m_textShapeData == 0)
         return;
+    connect(m_textShapeData, SIGNAL(destroyed (QObject*)), this, SLOT(shapeDataRemoved()));
     if (docChanged) {
         connect(m_textShapeData->document(), SIGNAL(undoCommandAdded()), this, SLOT(addUndoCommand()));
         m_caret = QTextCursor(m_textShapeData->document());
@@ -658,11 +672,11 @@ void TextTool::setShapeData(KoTextShapeData *data)
             m_textShape->setDemoText(false); // remove demo text
             m_textShapeData->document()->setUndoRedoEnabled(true); // allow undo history
         }
-    }
-    if (m_trackChanges) {
-        if (m_changeTracker == 0)
-            m_changeTracker = new ChangeTracker(this);
-        m_changeTracker->setDocument(m_textShapeData->document());
+        if (m_trackChanges) {
+            if (m_changeTracker == 0)
+                m_changeTracker = new ChangeTracker(this);
+            m_changeTracker->setDocument(m_textShapeData->document());
+        }
     }
     if (m_spellcheckPlugin)
         m_spellcheckPlugin->checkSection(m_textShapeData->document(), 0, 0);
@@ -861,7 +875,7 @@ void TextTool::keyPressEvent(QKeyEvent *event)
             editingPluginEvents();
         }
         ensureCursorVisible();
-    } else if ((event->key() == Qt::Key_Tab || event->key() == Qt::Key_BackTab)
+    } else if ((event->key() == Qt::Key_Tab || event->key() == Qt::Key_Backtab)
                && ((!m_caret.hasSelection() && (m_caret.position() == m_caret.block().position())) || (m_caret.block().document()->findBlock(m_caret.anchor()) != m_caret.block().document()->findBlock(m_caret.position()))) && m_caret.block().textList()) {
         ChangeListLevelCommand::CommandType type = 
             event->key() == Qt::Key_Tab ? ChangeListLevelCommand::IncreaseLevel : ChangeListLevelCommand::DecreaseLevel;
@@ -920,7 +934,7 @@ void TextTool::keyPressEvent(QKeyEvent *event)
             fn->setLabel("1");
             KoTextDocumentLayout *lay = dynamic_cast<KoTextDocumentLayout*>(m_textShapeData->document()->documentLayout());
             Q_ASSERT(lay);
-            lay->inlineObjectTextManager()->insertInlineObject(m_caret, fn);
+            lay->inlineTextObjectManager()->insertInlineObject(m_caret, fn);
         }
 #endif
         else if ((event->modifiers() & (Qt::ControlModifier | Qt::AltModifier)) || event->text().length() == 0) {
@@ -1044,7 +1058,11 @@ void TextTool::ensureCursorVisible()
             Q_ASSERT(textShape);
             KoTextShapeData *d = static_cast<KoTextShapeData*>(textShape->userData());
             if (m_caret.position() >= d->position() && m_caret.position() <= d->endPosition()) {
+                if (m_textShapeData)
+                    disconnect(m_textShapeData, SIGNAL(destroyed (QObject*)), this, SLOT(shapeDataRemoved()));
                 m_textShapeData = d;
+                if (m_textShapeData)
+                    connect(m_textShapeData, SIGNAL(destroyed (QObject*)), this, SLOT(shapeDataRemoved()));
                 m_textShape = textShape;
                 break;
             }
@@ -1502,13 +1520,13 @@ void TextTool::insertIndexMarker()
 
 void TextTool::formatParagraph()
 {
-    ParagraphSettingsDialog *dia = new ParagraphSettingsDialog(m_canvas->canvasWidget(), this);
-    dia->open(m_caret);
+    ParagraphSettingsDialog *dia = new ParagraphSettingsDialog(this, &m_caret);
     dia->setUnit(m_canvas->unit());
     connect(dia, SIGNAL(startMacro(const QString&)), this, SLOT(startMacro(const QString&)));
     connect(dia, SIGNAL(stopMacro()), this, SLOT(stopMacro()));
 
-    dia->show();
+    dia->exec();
+    delete dia;
 }
 
 void TextTool::toggleTrackChanges(bool on)
@@ -1525,7 +1543,8 @@ void TextTool::toggleTrackChanges(bool on)
 
 void TextTool::selectAll()
 {
-    if (m_textShapeData == 0) return;
+    if (m_textShapeData == 0)
+        return;
     const int selectionLength = qAbs(m_caret.position() - m_caret.anchor());
     QTextBlock lastBlock = m_textShapeData->document()->end().previous();
     m_caret.setPosition(lastBlock.position() + lastBlock.length() - 1);
@@ -1682,6 +1701,8 @@ void TextTool::insertSpecialCharacter()
 void TextTool::selectFont()
 {
     FontDia *fontDlg = new FontDia(&m_caret);
+    connect(fontDlg, SIGNAL(startMacro(const QString &)), this, SLOT(startMacro(const QString &)));
+    connect(fontDlg, SIGNAL(stopMacro()), this, SLOT(stopMacro()));
     fontDlg->exec();
     delete fontDlg;
 }
@@ -1701,6 +1722,25 @@ void TextTool::shapeAddedToCanvas()
     }
 }
 
+void TextTool::shapeDataRemoved()
+{
+    m_textShapeData = 0;
+    m_textShape = 0;
+    if (! m_caret.isNull()) {
+        const QTextDocument *doc = m_caret.block().document();
+        Q_ASSERT(doc);
+        KoTextDocumentLayout *lay = dynamic_cast<KoTextDocumentLayout*>(doc->documentLayout());
+        if (lay == 0 || lay->shapes().isEmpty()) {
+            emit done();
+            return;
+        }
+
+        m_textShape = static_cast<TextShape*>(lay->shapes().first());
+        m_textShapeData = static_cast<KoTextShapeData*>(m_textShape->userData());
+        connect(m_textShapeData, SIGNAL(destroyed (QObject*)), this, SLOT(shapeDataRemoved()));
+    }
+}
+
 // ---------- editing plugins methods.
 void TextTool::editingPluginEvents()
 {
@@ -1728,13 +1768,13 @@ void TextTool::editingPluginEvents()
 void TextTool::finishedWord()
 {
     foreach(KoTextEditingPlugin* plugin, m_textEditingPlugins)
-    plugin->finishedWord(m_textShapeData->document(), m_prevCursorPosition);
+        plugin->finishedWord(m_textShapeData->document(), m_prevCursorPosition);
 }
 
 void TextTool::finishedParagraph()
 {
     foreach(KoTextEditingPlugin* plugin, m_textEditingPlugins)
-    plugin->finishedParagraph(m_textShapeData->document(), m_prevCursorPosition);
+        plugin->finishedParagraph(m_textShapeData->document(), m_prevCursorPosition);
 }
 
 void TextTool::setTextColor(const KoColor &color)
@@ -1747,4 +1787,163 @@ void TextTool::setBackgroundColor(const KoColor &color)
     m_selectionHandler.setTextBackgroundColor(color.toQColor());
 }
 
+void TextTool::debugTextDocument()
+{
+#ifndef NDEBUG
+    if (m_textShapeData == 0)
+        return;
+    const int CHARSPERLINE = 80; // TODO Make configurable using ENV var?
+    const int CHARPOSITION = 278301935;
+    KoTextDocument document(m_textShapeData->document());
+    KoStyleManager *styleManager = document.styleManager();
+    KoInlineTextObjectManager *inlineManager = document.inlineTextObjectManager();
+
+    QTextBlock block = m_textShapeData->document()->begin();
+    for (;block.isValid(); block = block.next()) {
+        QVariant var = block.blockFormat().property(KoParagraphStyle::StyleId);
+        if (!var.isNull()) {
+            KoParagraphStyle *ps = styleManager->paragraphStyle(var.toInt());
+            kDebug(32500) << "--- Paragraph Style:" << (ps ? ps->name() : QString()) << var.toInt();
+        }
+        var = block.charFormat().property(KoCharacterStyle::StyleId);
+        if (!var.isNull()) {
+            KoCharacterStyle *cs = styleManager->characterStyle(var.toInt());
+            kDebug(32500) << "--- Character Style:" << (cs ? cs->name() : QString()) << var.toInt();
+        }
+        int lastPrintedChar = -1;
+        QTextBlock::iterator it;
+        QString fragmentText;
+        QList<QTextCharFormat> inlineCharacters;
+        for (it = block.begin(); !it.atEnd(); ++it) {
+            QTextFragment fragment = it.fragment();
+            if (!fragment.isValid())
+                continue;
+            QTextCharFormat fmt = fragment.charFormat();
+            const int fragmentStart = fragment.position() - block.position();
+            for (int i = fragmentStart; i < fragmentStart + fragment.length(); i += CHARSPERLINE) {
+                if (lastPrintedChar == fragmentStart-1)
+                    fragmentText += '|';
+                if (lastPrintedChar < fragmentStart || i > fragmentStart) {
+                    QString debug = block.text().mid(lastPrintedChar, CHARSPERLINE);
+                    lastPrintedChar += CHARSPERLINE;
+                    if (lastPrintedChar > block.length())
+                        debug += "\\n";
+                    kDebug(32500) << debug;
+                }
+                var = fmt.property(KoCharacterStyle::StyleId);
+                QString charStyleLong, charStyleShort;
+                if (! var.isNull()) { // named style
+                    charStyleShort = QString::number(var.toInt());
+                    KoCharacterStyle *cs = styleManager->characterStyle(var.toInt());
+                    if (cs)
+                        charStyleLong = cs->name();
+                }
+                if (inlineManager && fmt.hasProperty(KoCharacterStyle::InlineInstanceId)) {
+                    QTextCharFormat inlineFmt = fmt;
+                    inlineFmt.setProperty(CHARPOSITION, fragmentStart);
+                    inlineCharacters << inlineFmt;
+                }
+
+                if (fragment.length() > charStyleLong.length())
+                    fragmentText += charStyleLong;
+                else if (fragment.length() > charStyleShort.length())
+                    fragmentText += charStyleShort;
+                else if (fragment.length() >= 2)
+                    fragmentText += QChar(8230); // elipses
+
+
+
+                int rest =  fragmentStart - (lastPrintedChar-CHARSPERLINE) + fragment.length() - fragmentText.length();
+                rest = qMin(rest, CHARSPERLINE - fragmentText.length());
+                if (rest >= 2)
+                    fragmentText = QString("%1%2").arg(fragmentText).arg(' ', rest);
+                if (rest >= 0)
+                    fragmentText += '|';
+                if (fragmentText.length() >= CHARSPERLINE) {
+                    kDebug(32500) << fragmentText;
+                    fragmentText.clear();
+                }
+            }
+        }
+        if (!fragmentText.isEmpty())
+            kDebug(32500) << fragmentText;
+        else if (block.length() == 1) // no actual tet
+            kDebug(32500) << "\\n";
+        foreach (QTextCharFormat cf, inlineCharacters) {
+            KoInlineObject *object= inlineManager->inlineTextObject(cf);
+            kDebug(32500) << "At pos:" << cf.intProperty(CHARPOSITION) << object;
+            // kDebug(32500) << "-> id:" << cf.intProperty(577297549);
+        }
+        QTextList *list = block.textList();
+        if (list) {
+            if (list->format().hasProperty(KoListStyle::StyleId)) {
+                KoListStyle *ls = styleManager->listStyle(list->format().intProperty(KoListStyle::StyleId));
+                kDebug(32500) << "   List style applied:" << ls->styleId() << ls->name();
+            }
+            else
+                kDebug(32500) << " +- is a list..." << list;
+        }
+    }
+#endif
+}
+
+void TextTool::debugTextStyles()
+{
+#ifndef NDEBUG
+    KoTextDocument document(m_textShapeData->document());
+    KoStyleManager *styleManager = document.styleManager();
+
+    QSet<int> seenStyles;
+
+    foreach (KoParagraphStyle *style, styleManager->paragraphStyles()) {
+        kDebug(32500) << style->styleId() << style->name() << (styleManager->defaultParagraphStyle() == style ? "[Default]" : "");
+        KoCharacterStyle *cs = style->characterStyle();
+        seenStyles << style->styleId();
+        if (cs) {
+            kDebug(32500) << "  +- CharStyle: " << cs->styleId() << cs->name();
+            kDebug(32500) << "  |  " << cs->font();
+            seenStyles << cs->styleId();
+        } else {
+            kDebug(32500) << "  +- ERROR; no char style found!" << endl;
+        }
+        KoListStyle *ls = style->listStyle();
+        if (ls) { // optional ;)
+            kDebug(32500) << "  +- ListStyle: " << ls->styleId() << ls->name()
+                << (ls == styleManager->defaultListStyle() ? "[Default]":"");
+            foreach (int level, ls->listLevels()) {
+                KoListLevelProperties llp = ls->levelProperties(level);
+                kDebug(32500) << "  |  level" << llp.level() << " style (enum):" << llp.style();
+                if (llp.bulletCharacter().unicode() != 0)
+                    kDebug(32500) << "  |  bullet" << llp.bulletCharacter();
+            }
+            seenStyles << ls->styleId();
+        }
+    }
+
+    bool first = true;
+    foreach (KoCharacterStyle *style, styleManager->characterStyles()) {
+        if (seenStyles.contains(style->styleId()))
+            continue;
+        if (first) {
+            kDebug(32500) << "--- Character styles ---";
+            first = false;
+        }
+        kDebug(32500) << style->styleId() << style->name();
+        kDebug(32500) << style->font();
+    }
+
+    first = true;
+    foreach (KoListStyle *style, styleManager->listStyles()) {
+        if (seenStyles.contains(style->styleId()))
+            continue;
+        if (first) {
+            kDebug(32500) << "--- List styles ---";
+            first = false;
+        }
+        kDebug(32500) << style->styleId() << style->name()
+                << (style == styleManager->defaultListStyle() ? "[Default]":"");
+    }
+#endif
+}
+
 #include "TextTool.moc"
